<?php
/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

namespace Shopgate\Base\Api\Config;

interface SgCoreInterface
{
    const SECTION_BASE         = 'shopgate_base';
    const SECTION_ADVANCED     = 'shopgate_advanced';
    const PATH_GENERAL         = self::SECTION_BASE . '/general';
    const PATH_REDIRECT        = self::SECTION_BASE . '/redirect';
    const PATH_ACTIVE          = self::PATH_GENERAL . '/active';
    const PATH_CUSTOMER_NUMBER = self::PATH_GENERAL . '/customer_number';
    const PATH_SHOP_NUMBER     = self::PATH_GENERAL . '/shop_number';
    const PATH_API_KEY         = self::PATH_GENERAL . '/api_key';
    const PATH_ALIAS           = self::PATH_REDIRECT . '/alias';
    const PATH_CNAME           = self::PATH_REDIRECT . '/cname';
    const PATH_REDIRECT_TYPE   = self::PATH_REDIRECT . '/type';
    const PATH_CMS_MAP         = self::PATH_REDIRECT . '/cms_map';
    const PATH_HTUSER          = self::SECTION_ADVANCED . '/staging/htuser';
    const PATH_HTPASS          = self::SECTION_ADVANCED . '/staging/htpass';
    const PATH_SERVER_TYPE     = self::SECTION_ADVANCED . '/staging/server';
    const PATH_API_URL         = self::SECTION_ADVANCED . '/staging/api_url';

    const VALUE_REDIRECT_HTTP = 'http';
    const VALUE_REDIRECT_JS   = 'js';

    const VALUE_SERVER_LIVE   = 'live';
    const VALUE_SERVER_PG     = 'pg';
    const VALUE_SERVER_CUSTOM = 'custom';

    /**
     * Retrieves the store ID based on Shopgate
     * shop number provided by the Merchant API
     *
     * @param string $shopNumber
     *
     * @return int
     */
    public function getStoreId($shopNumber);

    /**
     * Retrieves the Shopnumber collection
     *
     * @param string $shopNumber
     *
     * @return \Magento\Config\Model\ResourceModel\Config\Data\Collection
     */
    public function getShopNumberCollection($shopNumber);

    /**
     * Sets values onto stores scope if a websites scope value is threatening to overwrite them
     *
     * @param string $path
     * @param string $shopNumber
     *
     * @return \Magento\Framework\App\Config\Value
     */
    public function getSaveScope($path, $shopNumber);

    /**
     * Checks if the current store's vital
     * configurations are set:
     * 1) Enabled
     * 2) API Key
     * 3) Shop Number
     * 4) Customer Number
     *
     * @return bool
     */
    public function isValid();
}
