<?php

/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

namespace Shopgate\Base\Api;

use Shopgate\Base\Model\Shopgate\Extended\Base;

/**
 * Main export interface functions
 */
interface ExportInterface
{
    const SECTION_EXPORT             = 'shopgate_export';
    const PATH_CATEGORIES            = self::SECTION_EXPORT . '/categories';
    const PATH_PRODUCTS              = self::SECTION_EXPORT . '/products';
    const PATH_MISC                  = self::SECTION_EXPORT . '/misc';
    const PATH_PROD_EAN_CODE         = self::PATH_PRODUCTS . '/ean_code';
    const PATH_PROD_FORCE_ATTRIBUTES = self::PATH_PRODUCTS . '/force_property_export';

    /**
     * Returns JSON to comply with Mage 2 WebApi
     *
     * @link http://developer.shopgate.com/plugin_api/export/get_categories
     *
     * @param string $action     - Requested method, e.g. get_categories
     * @param string $shopNumber - Your shop number as configured in the merchant API
     * @param string $traceId    - Unique request trace ID, handshake related
     * @param string $limit      - limit number of categories to pull
     * @param string $offset     - the offset number to pull from, e.g. 10 means next $limit from 10th item
     * @param array  $uids       - array of category ID's to pull
     *
     * @return string
     */
    public function getCategories($action, $shopNumber, $traceId, $limit = null, $offset = null, array $uids = []);

    /**
     * Raw data retriever for regular Merchant API requests
     *
     * @link http://developer.shopgate.com/plugin_api/export/get_categories
     *
     * @param null  $limit  - limit number of categories to pull
     * @param null  $offset - the offset number to pull from, e.g. 10 means next $limit from 10th item
     * @param array $uids   - array of category ID's to pull
     *
     * @return \Shopgate\Export\Model\Export\Category[]
     */
    public function getCategoriesRaw($limit = null, $offset = null, array $uids = []);

    /**
     * Returns JSON to comply with Mage 2 WebApi
     *
     * @link http://developer.shopgate.com/plugin_api/export/get_items
     *
     * @param string $action     - Requested method, e.g. get_items
     * @param string $shopNumber - Your shop number as configured in the merchant API
     * @param string $traceId    - Unique request trace ID, handshake related
     * @param string $limit      - limit number of categories to pull
     * @param string $offset     - the offset number to pull from, e.g. 10 means next $limit from 10th item
     * @param array  $uids       - array of category ID's to pull
     *
     * @return string
     */
    public function getItems($action, $shopNumber, $traceId, $limit = null, $offset = null, array $uids = []);

    /**
     * Raw data retriever for regular Merchant API requests
     *
     * @link http://developer.shopgate.com/plugin_api/export/get_items
     *
     * @param null  $limit  - limit number of categories to pull
     * @param null  $offset - the offset number to pull from, e.g. 10 means next $limit from 10th item
     * @param array $uids   - array of category ID's to pull
     *
     * @return \Shopgate\Export\Model\Export\Product[]
     */
    public function getItemsRaw($limit = null, $offset = null, array $uids = []);

    /**
     * Returns the customer data, if the credentials are valid
     *
     * @link http://developer.shopgate.com/plugin_api/customers/get_customer
     *
     * @param string $user - The user name the customer entered at Shopgate Connect.
     * @param string $pass - The password the customer entered at Shopgate Connect.
     *
     * @return string
     * @throws \ShopgateLibraryException
     */
    public function getCustomer($user, $pass);

    /**
     * Returns the customer data for regular Merchant API requests
     *
     * @link http://developer.shopgate.com/plugin_api/customers/get_customer
     *
     * @param string $user - The user name the customer entered at Shopgate Connect.
     * @param string $pass - The password the customer entered at Shopgate Connect.
     *
     * @return \ShopgateCustomer
     * @throws \ShopgateLibraryException
     */
    public function getCustomerRaw($user, $pass);

    /**
     * Returns the check_cart data for magento API service.
     * Only allowed methods inside the DI.xml will be returned.
     *
     * @link http://developer.shopgate.com/plugin_api/cart
     *
     * @param array $cart
     *
     * @return string
     */
    public function checkCart(array $cart);

    /**
     * Returns the check_cart data for regular Merchant API requests.
     * Only allowed methods inside the DI.xml will be returned.
     *
     * @link http://developer.shopgate.com/plugin_api/cart
     *
     * @param Base | \ShopgateCart $cart
     *
     * @return array
     */
    public function checkCartRaw($cart);
}
