<?php

/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

namespace Shopgate\Base\Api;

use Shopgate\Base\Model\Shopgate\Extended\Base;
use ShopgateCustomer;

interface ImportInterface
{
    /**
     * Registers a customer using magento API
     *
     * @link http://developer.shopgate.com/plugin_api/customers/register_customer
     *
     * @param string $action     - requested method, "register_customer" in this case
     * @param string $shopNumber - your shop number as configured in the merchant API
     * @param string $user       - email address a customer used to register a new account
     * @param string $pass       - the password a customer used to register a new account
     * @param string $traceId    - unique request trace ID, handshake related
     * @param array  $userData   - user specific data, like address, etc
     */
    public function registerCustomer($action, $shopNumber, $user, $pass, $traceId, $userData);

    /**
     * Registers a customer using the Shopgate Merchant API call
     *
     * @link http://developer.shopgate.com/plugin_api/customers/register_customer
     *
     * @param string           $user     - email address a customer used to register a new account
     * @param string           $pass     - the password a customer used to register a new account
     * @param ShopgateCustomer $customer - customer object
     */
    public function registerCustomerRaw($user, $pass, ShopgateCustomer $customer);

    /**
     * Performs the necessary queries to add an order to the shop system's database.
     *
     * @see http://developer.shopgate.com/merchant_api/orders/get_orders
     * @see http://developer.shopgate.com/plugin_api/orders/add_order
     *
     * @param Base | \ShopgateOrder $order The ShopgateOrder object to be added to the shop system's database.
     *
     * @return array(
     *          <ul>
     *            <li>'external_order_id' => <i>string</i>, # the actual order ID as in database</li>
     *              <li>'external_order_number' => <i>string</i> # the increment ID of the order</li>
     *          </ul>)
     * @throws \ShopgateLibraryException if an error occurs.
     */
    public function addOrder($order);

    /**
     * Performs the necessary queries to update an order in the shop system's database.
     *
     * @see http://developer.shopgate.com/merchant_api/orders/get_orders
     * @see http://developer.shopgate.com/plugin_api/orders/update_order
     *
     * @param \ShopgateOrder $order The ShopgateOrder object to be updated in the shop system's database.
     *
     * @return array(
     *          <ul>
     *              <li>'external_order_id' => <i>string</i>, # the ID of the order in your shop system's database</li>
     *              <li>'external_order_number' => <i>string</i> # the number of the order in your shop system</li>
     *          </ul>)
     * @throws \ShopgateLibraryException if an error occurs.
     */
    public function updateOrder($order);
}
