<?php
/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

namespace Shopgate\Base\Block\Adminhtml\Form\Field;

use Magento\Cms\Model\PageFactory;
use Magento\Framework\View\Element\Context;
use Magento\Framework\View\Element\Html\Select;
use Magento\Store\Model\StoreManagerInterface;
use Magento\Store\Model\Website;

class CmsMap extends Select
{
    /** @var PageFactory */
    protected $cmsFactory;
    /** @var StoreManagerInterface */
    private $storeManager;

    /**
     * @param Context               $context
     * @param PageFactory           $cmsFactory
     * @param StoreManagerInterface $storeManager
     * @param array                 $data
     */
    public function __construct(
        Context $context,
        PageFactory $cmsFactory,
        StoreManagerInterface $storeManager,
        array $data = []
    ) {
        parent::__construct($context, $data);
        $this->cmsFactory   = $cmsFactory;
        $this->storeManager = $storeManager;
    }

    /**
     * Sets name for input element
     *
     * @param string $value
     *
     * @return $this
     */
    public function setInputName($value)
    {
        return $this->setData('name', $value);
    }

    /**
     * Retrieves all the pages that are allowed to be
     * viewed in the current context
     *
     * @return string
     * @throws \Magento\Framework\Exception\LocalizedException
     * @codingStandardsIgnoreStart
     */
    protected function _toHtml()
    {
        //@codingStandardsIgnoreEnd
        if (!$this->getOptions()) {
            $cmsPages = $this->cmsFactory
                ->create()
                ->getCollection()
                ->join('cms_page_store', 'main_table.page_id  = cms_page_store.page_id')
                ->addFieldToFilter(
                    ['store_id', 'store_id'],
                    [
                        ['in' => $this->getStoreFromContext()],
                        ['eq' => 0]
                    ]
                );
            foreach ($cmsPages as $cmsPage) {
                /** @var \Magento\Cms\Model\Page $cmsPage */
                $this->addOption($cmsPage->getId(), $cmsPage->getTitle());
            }
        }

        return parent::_toHtml();
    }

    /**
     * Retrieves the stores that are allowed in the context
     * E.g. Store X will just return itself
     * E.g. Website Y will return an array of all stores under it
     * E.g. Default will return all store ids
     *
     * @return array - e.g. [1] or [1,3]
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function getStoreFromContext()
    {
        $params = $this->getRequest()->getParams();

        if (isset($params['store'])) {
            return [$params['store']];
        } elseif (isset($params['website'])) {
            /** @var Website $website */
            $website = $this->storeManager->getWebsite($params['website']);

            return $website->getStoreIds();
        }

        return array_keys($this->storeManager->getStores());
    }
}
