<?php
/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

namespace Shopgate\Base\Helper\Initializer;

use Magento\Config\Model\ResourceModel\Config as ConfigResource;
use Magento\Framework\App\CacheInterface;
use Magento\Framework\App\Filesystem\DirectoryList;
use Magento\Framework\App\RequestInterface;
use Magento\Store\Model\StoreManagerInterface;
use Shopgate\Base\Api\Config\CoreInterface;
use Shopgate\Base\Api\Config\SgCoreInterface;
use Shopgate\Base\Model\Utility\SgLoggerInterface;

/**
 * The main purpose of this class is to initialize classes and pass them along.
 * This hack is needed due to the non-rewritable final constructor on ShopgateConfig lib class
 */
class Config
{
    /** @var StoreManagerInterface */
    private $storeManager;
    /** @var SgLoggerInterface */
    private $logger;
    /** @var DirectoryList */
    private $directory;
    /** @var CoreInterface */
    protected $coreConfig;
    /** @var sgCoreInterface */
    private $sgCoreConfig;
    /** @var CacheInterface */
    private $cache;
    /** @var ConfigResource */
    private $configResource;
    /** @var array - see global di.xml */
    private $configMapping;
    /** @var array - see global di.xml */
    private $configMethods;
    /** @var RequestInterface */
    private $request;

    /**
     * @param StoreManagerInterface $storeManager
     * @param SgLoggerInterface     $logger
     * @param DirectoryList         $directory
     * @param SgCoreInterface       $sgCoreConfig
     * @param CoreInterface         $coreConfig
     * @param CacheInterface        $cache
     * @param ConfigResource        $configResource
     * @param RequestInterface      $request
     * @param array                 $configMethods - config methods to enable
     * @param array                 $configMapping - configs to retrieve from the database
     *
     * @codeCoverageIgnore
     */
    public function __construct(
        StoreManagerInterface $storeManager,
        SgLoggerInterface $logger,
        DirectoryList $directory,
        SgCoreInterface $sgCoreConfig,
        CoreInterface $coreConfig,
        CacheInterface $cache,
        ConfigResource $configResource,
        RequestInterface $request,
        array $configMethods = [],
        array $configMapping = []
    ) {
        $this->configMapping  = $configMapping;
        $this->storeManager   = $storeManager;
        $this->logger         = $logger;
        $this->directory      = $directory;
        $this->configMapping  = $configMapping;
        $this->sgCoreConfig   = $sgCoreConfig;
        $this->coreConfig     = $coreConfig;
        $this->cache          = $cache;
        $this->configResource = $configResource;
        $this->configMethods  = $configMethods;
        $this->request        = $request;
    }

    /**
     * @codeCoverageIgnore
     * @return StoreManagerInterface
     */
    public function getStoreManager()
    {
        return $this->storeManager;
    }

    /**
     * @codeCoverageIgnore
     * @return SgLoggerInterface
     */
    public function getLogger()
    {
        return $this->logger;
    }

    /**
     * @codeCoverageIgnore
     * @return DirectoryList
     */
    public function getDirectory()
    {
        return $this->directory;
    }

    /**
     * @codeCoverageIgnore
     * @return array
     */
    public function getConfigMapping()
    {
        return $this->configMapping;
    }

    /**
     * @codeCoverageIgnore
     * @return CoreInterface
     */
    public function getCoreConfig()
    {
        return $this->coreConfig;
    }

    /**
     * @codeCoverageIgnore
     * @return SgCoreInterface
     */
    public function getSgCoreConfig()
    {
        return $this->sgCoreConfig;
    }

    /**
     * @codeCoverageIgnore
     * @return CacheInterface
     */
    public function getCache()
    {
        return $this->cache;
    }

    /**
     * @codeCoverageIgnore
     * @return ConfigResource
     */
    public function getConfigResource()
    {
        return $this->configResource;
    }

    /**
     * @codeCoverageIgnore
     * @return array
     */
    public function getConfigMethods()
    {
        return $this->configMethods;
    }

    /**
     * @return RequestInterface
     */
    public function getRequest()
    {
        return $this->request;
    }
}
