<?php

/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

namespace Shopgate\Base\Helper\Product;

use Magento\Bundle\Model\Product\Type as Bundle;
use Magento\Catalog\Model\Product;
use Magento\ConfigurableProduct\Model\Product\Type\Configurable;
use Magento\GroupedProduct\Model\Product\Type\Grouped;
use Magento\Quote\Model\Quote\Item as QuoteItem;
use Shopgate\Base\Helper\Product\Type\Exception;
use Shopgate\Base\Model\Shopgate\Extended\OrderItem;

class Type
{
    /** @var Type\Bundle */
    private $bundle;
    /** @var Type\Generic */
    private $generic;
    /** @var Type\Grouped */
    private $grouped;
    /** @var Type\Configurable */
    private $configurable;

    /**
     * @param Type\BundleFactory       $bundle
     * @param Type\GenericFactory      $generic
     * @param Type\GroupedFactory      $grouped
     * @param Type\ConfigurableFactory $configurable
     */
    public function __construct(
        Type\BundleFactory $bundle,
        Type\GenericFactory $generic,
        Type\GroupedFactory $grouped,
        Type\ConfigurableFactory $configurable
    ) {
        $this->bundle       = $bundle;
        $this->grouped      = $grouped;
        $this->generic      = $generic;
        $this->configurable = $configurable;
    }

    /**
     * Retrieves the correct type of product helper
     * based on passed item object
     *
     * @param OrderItem | Product | QuoteItem $item
     *
     * @return Type\Bundle | Type\Generic | Type\Grouped | Type\Configurable
     * @throws \Exception
     */
    public function getType($item)
    {
        switch ($this->getProductType($item)) {
            case Bundle::TYPE_CODE:
                return $this->bundle->create()->setItem($item);
            case Grouped::TYPE_CODE:
                return $this->grouped->create()->setItem($item);
            case Configurable::TYPE_CODE:
                return $this->configurable->create()->setItem($item);
            default:
                return $this->generic->create()->setItem($item);
        }
    }

    /**
     * Determine the product's type based on the
     * item/product passed
     *
     * @param OrderItem | Product | QuoteItem $item
     *
     * @return string - e.g. "bundle", "configurable", "simple", "grouped"
     * @throws \Exception
     */
    private function getProductType($item)
    {
        $type = '';
        if ($item instanceof OrderItem) {
            $type = $item->getInternalOrderInfo()->getItemType();
        } elseif ($item instanceof Product) {
            $type = $item->getTypeId();
        } elseif ($item instanceof QuoteItem) {
            $type = $item->getProduct()->getTypeId();
        }

        if (empty($type)) {
            throw new Exception('Did not recognize the passed object: ' . get_class($item));
        }

        return $type;
    }
}
