<?php

/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

namespace Shopgate\Base\Helper\Product;

use Magento\Catalog\Api\CategoryRepositoryInterface;
use Magento\Catalog\Api\ProductRepositoryInterface;
use Magento\Catalog\Model\Product as MageProduct;
use Magento\Catalog\Model\Product\Type as ProductType;
use Magento\CatalogInventory\Api\Data\StockItemInterfaceFactory;
use Magento\CatalogInventory\Model\ResourceModel\Stock\Item as StockItemResource;
use Magento\Cms\Model\Template\FilterProvider;
use Magento\Framework\DataObject;
use Magento\Store\Model\StoreManager;
use Magento\Tax\Model\Config as TaxConfig;
use Magento\Tax\Model\TaxCalculation;
use Shopgate\Base\Api\Config\CoreInterface;
use Shopgate\Base\Model\Shopgate\Extended\OrderItem;
use Shopgate\Export\Model\Export\Utility as ExportUtility;

class Utility
{
    /** @var StockItemResource */
    protected $stockItemResource;
    /** @var StockItemInterfaceFactory */
    protected $stockItemFactory;
    /** @var StoreManager */
    protected $storeManager;
    /** @var TaxCalculation */
    protected $taxCalculation;
    /** @var ExportUtility */
    protected $utility;
    /** @var TaxConfig */
    protected $taxConfig;
    /** @var Type */
    private $typeHelper;
    /** @var Type\Grouped */
    private $grouped;
    /** @var ProductRepositoryInterface */
    protected $productRepository;
    /** @var CategoryRepositoryInterface */
    protected $categoryRepository;
    /** @var FilterProvider */
    protected $filter;
    /** @var CoreInterface */
    protected $sgCore;

    /**
     * @param StockItemResource           $stockItemResource
     * @param StockItemInterfaceFactory   $stockItemFactory
     * @param StoreManager                $storeManager
     * @param TaxCalculation              $taxCalculation
     * @param ExportUtility               $utility
     * @param ProductRepositoryInterface  $productRepository
     * @param CategoryRepositoryInterface $categoryRepository
     * @param TaxConfig                   $taxConfig
     * @param Type\Grouped                $grouped
     * @param Type                        $typeHelper
     * @param FilterProvider              $filter
     * @param CoreInterface               $sgCore
     */
    public function __construct(
        StockItemResource $stockItemResource,
        StockItemInterfaceFactory $stockItemFactory,
        StoreManager $storeManager,
        TaxCalculation $taxCalculation,
        ExportUtility $utility,
        ProductRepositoryInterface $productRepository,
        CategoryRepositoryInterface $categoryRepository,
        TaxConfig $taxConfig,
        Type\Grouped $grouped,
        Type $typeHelper,
        FilterProvider $filter,
        CoreInterface $sgCore
    ) {
        $this->stockItemResource  = $stockItemResource;
        $this->stockItemFactory   = $stockItemFactory;
        $this->storeManager       = $storeManager;
        $this->taxCalculation     = $taxCalculation;
        $this->utility            = $utility;
        $this->taxConfig          = $taxConfig;
        $this->grouped            = $grouped;
        $this->typeHelper         = $typeHelper;
        $this->productRepository  = $productRepository;
        $this->categoryRepository = $categoryRepository;
        $this->filter             = $filter;
        $this->sgCore             = $sgCore;
    }

    /**
     * @param string | int $productId
     *
     * @return \Magento\Catalog\Api\Data\ProductInterface|MageProduct
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function loadById($productId)
    {
        return $this->productRepository->getById($productId, false, $this->storeManager->getStore()->getId());
    }

    /**
     * @param OrderItem   $item
     * @param MageProduct $product
     *
     * @return DataObject
     * @throws \Exception
     */
    public function getBuyInfo(OrderItem $item, MageProduct $product)
    {
        $orderInfo     = $item->getInternalOrderInfo();
        $buyInfo       = new DataObject(
            [
                'qty'     => $item->getQuantity() * $item->getStackQty(),
                'product' => $product->getId(),
            ]
        );

        if ($orderInfo->getOptions()) {
            $buyInfo->setData('super_attribute', $orderInfo->getOptions());
        }

        $options = $this->typeHelper->getType($item)->parseOptions($product);

        foreach ($item->getInputs() as $orderInput) {
            $options['options'][$orderInput->getInputNumber()] = $orderInput->getUserInput();
        }

        return $buyInfo->addData($options);
    }

    /**
     * @return Type\Grouped
     */
    public function getGrouped()
    {
        return $this->grouped;
    }
}
