<?php

/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

namespace Shopgate\Base\Helper\Quote;

use Magento\Framework\Exception\CouldNotSaveException;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Quote\Model\CouponManagement;
use Magento\Quote\Model\Quote;
use Shopgate\Base\Model\Shopgate\Extended\Base;
use Shopgate\Base\Model\Shopgate\Extended\ExternalCoupon;
use Shopgate\Base\Model\Utility\SgLoggerInterface;
use ShopgateLibraryException;

class Coupon
{
    /** @var Quote */
    private $quote;
    /** @var Base */
    private $cart;
    /** @var SgLoggerInterface */
    private $log;
    /** @var CouponManagement */
    private $couponManagement;
    /** @var int - counts the amount of valid coupons in cart */
    private $validCoupons = 0;

    /**
     * @param Quote             $quote
     * @param Base              $cart
     * @param SgLoggerInterface $log
     * @param CouponManagement  $couponManagement
     */
    public function __construct(
        Quote $quote,
        Base $cart,
        SgLoggerInterface $log,
        CouponManagement $couponManagement
    ) {
        $this->quote            = $quote;
        $this->cart             = $cart;
        $this->log              = $log;
        $this->couponManagement = $couponManagement;
    }

    /**
     * Sets coupon to quote
     */
    public function setCoupon()
    {
        foreach ($this->cart->getExternalCoupons() as $coupon) {
            $this->setCouponToQuote($coupon);
        }
    }

    /**
     * @param ExternalCoupon $coupon
     *
     * @throws \Exception
     */
    public function setCouponToQuote(ExternalCoupon $coupon)
    {
        $couponId = $this->quote->getEntityId();
        try {
            if ($this->couponManagement->set($couponId, $coupon->getCode()) && $this->validCoupons > 0) {
                throw new ShopgateLibraryException(ShopgateLibraryException::COUPON_TOO_MANY_COUPONS);
            }
            $this->quote->loadActive($couponId);
            $this->validCoupons++;
        } catch (ShopgateLibraryException $e) {
            $coupon->setErrorByCode($e->getCode());
        } catch (NoSuchEntityException $e) {
            $this->log->debug($e->getMessage());
            $coupon->setErrorByCode(ShopgateLibraryException::COUPON_NOT_VALID);
        } catch (CouldNotSaveException $e) {
            $coupon->setErrorByCode(ShopgateLibraryException::COUPON_NOT_VALID);
        } catch (\Exception $e) {
            $this->log->debug($e->getMessage());
            $coupon->setErrorByCode(ShopgateLibraryException::UNKNOWN_ERROR_CODE);
        }
    }
}
