<?php

/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */
namespace Shopgate\Base\Helper\Quote;

use Magento\Customer\Model\Session;
use Magento\Framework\DataObject;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Quote\Model\Quote as MageQuote;
use Shopgate\Base\Helper\Customer as CustomerHelper;
use Shopgate\Base\Helper\Shopgate\Customer as SgCustomerHelper;
use Shopgate\Base\Model\Shopgate\Extended\Base;
use Shopgate\Base\Model\Utility\SgLoggerInterface;

/**
 * Class that helps set the customer object to quote and session
 */
class Customer
{
    /** @var Base */
    private $sgBase;
    /** @var CustomerHelper */
    private $customerHelper;
    /** @var Session */
    private $session;
    /** @var SgCustomerHelper */
    private $sgCustomer;
    /** @var MageQuote */
    private $quote;
    /** @var SgLoggerInterface */
    private $log;

    /**
     * @param Base              $sgBase
     * @param CustomerHelper    $customerHelper
     * @param MageQuote         $quote
     * @param SgCustomerHelper  $sgCustomerHelper
     * @param Session           $session
     * @param SgLoggerInterface $log
     */
    public function __construct(
        Base $sgBase,
        CustomerHelper $customerHelper,
        MageQuote $quote,
        Session $session,
        SgCustomerHelper $sgCustomerHelper,
        SgLoggerInterface $log
    ) {
        $this->sgBase         = $sgBase;
        $this->customerHelper = $customerHelper;
        $this->session        = $session;
        $this->sgCustomer     = $sgCustomerHelper;
        $this->quote          = $quote;
        $this->log            = $log;
    }

    /**
     * Just sets the customer to quote
     */
    public function setEntity()
    {
        $id    = $this->sgBase->getExternalCustomerId();
        $email = $this->sgBase->getMail();

        try {
            $customer = $id ? $this->customerHelper->getById($id) : $this->customerHelper->getByEmail($email);
        } catch (NoSuchEntityException $e) {
            $customer = new DataObject();
            $this->log->debug('Could not load customer by id or mail.');
        }
        $this->quote->setCustomerEmail($email);
        if ($this->sgBase->isGuest()) {
            $this->quote->setCustomerIsGuest(true);
        } elseif (!$this->sgBase->isGuest() && $customer->getId()) {
            $this->session->setCustomerId($customer->getId())->setCustomerGroupId($customer->getGroupId());

            $this->quote->setCustomer($customer)
                        ->setRemoteIp($this->sgBase->getCustomerIp())
                        ->setCustomerIsGuest(false);
        } else {
            throw new \ShopgateLibraryException(
                \ShopgateLibraryException::UNKNOWN_ERROR_CODE,
                __('Customer with external id "%1" or email "%2" does not exist', $id, $email)->render()
            );
        }
    }

    /**
     * Sets Billing and Shipping addresses to quote
     */
    public function setAddress()
    {
        $billing = $this->sgBase->getInvoiceAddress();
        if (!empty($billing)) {
            //$this->quote->getBillingAddress()->setData('should_ignore_validation', true); //todo-sg: cannot find check
            $data = $this->sgCustomer->createAddressData($this->sgBase, $billing, true);
            $this->quote->getBillingAddress()->addData($data)->setCustomerAddressId($billing->getId());
        }

        $shipping = $this->sgBase->getDeliveryAddress();
        if (!empty($shipping)) {
            //$this->quote->getShippingAddress()->setData('should_ignore_validation', true);
            $data = $this->sgCustomer->createAddressData($this->sgBase, $shipping);
            $this->quote->getShippingAddress()->addData($data)->setCustomerAddressId($shipping->getId());
        }
    }

    /**
     * Helps reset the billing & shipping objects for guests
     *
     * @see https://shopgate.atlassian.net/browse/MAGENTO-429
     *
     */
    public function resetGuest()
    {
        if ($this->sgBase->isGuest()) {
            $this->quote->getBillingAddress()->isObjectNew(false);
            $this->quote->getShippingAddress()->isObjectNew(false);
        }
    }
}
