<?php

/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

namespace Shopgate\Base\Helper;

use Magento\Backend\App\Action\Context;
use Magento\Backend\App\Area\FrontNameResolver;
use Magento\Framework\Model\Context as LegacyContext;
use Shopgate\Base\Api\Config\CoreInterface;
use Shopgate\Base\Api\Config\SgCoreInterface;

class Redirect
{
    /** @var LegacyContext */
    private $legacyContext;
    /** @var SgCoreInterface */
    private $sgConfig;
    /** @var Context */
    private $context;
    /** @var CoreInterface */
    private $coreConfig;

    /**
     * @param LegacyContext   $legacyContext
     * @param CoreInterface   $coreConfig
     * @param SgCoreInterface $sgConfig
     * @param Context         $context
     *
     * @codeCoverageIgnore
     */
    public function __construct(
        LegacyContext $legacyContext,
        CoreInterface $coreConfig,
        SgCoreInterface $sgConfig,
        Context $context
    ) {
        $this->legacyContext = $legacyContext;
        $this->sgConfig      = $sgConfig;
        $this->context       = $context;
        $this->coreConfig    = $coreConfig;
    }

    /**
     * Redirect true if
     * 1) Not admin page request
     * 2) Not an ajax request
     * 3) SG Config is valid (active, api_key, shop_number, customer_number)
     * 4) SG Module enabled for this store
     *
     * @return bool
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function isAllowed()
    {
        /** @var \Magento\Framework\App\Request\Http $req */
        $req = $this->context->getRequest();

        if ($this->legacyContext->getAppState()->getAreaCode() !== FrontNameResolver::AREA_CODE
            && !$req->isAjax()
            && $this->sgConfig->isValid()
        ) {
            return true;
        }

        return false;
    }

    /**
     * Checks if setting for JavaScript type redirect is set
     * in core_config_data table
     *
     * @return bool
     */
    public function isTypeJavaScript()
    {
        $redirectType = $this->coreConfig->getConfigByPath(SgCoreInterface::PATH_REDIRECT_TYPE);

        return $redirectType->getValue() === SgCoreInterface::VALUE_REDIRECT_JS;
    }
}
