<?php
/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

namespace Shopgate\Base\Helper\Settings\Payment;

use Magento\Payment\Model\MethodList;
use Magento\Quote\Api\Data\CartInterface;
use Magento\Store\Model\StoreManagerInterface;

class Retriever
{

    /**
     * @var StoreManagerInterface
     */
    private $storeManager;
    /**
     * @var CartInterface
     */
    private $quote;
    /**
     * @var MethodList
     */
    private $paymentModel;

    /**
     * @param StoreManagerInterface $storeManager - store id
     * @param CartInterface         $quote        - fake quote object
     * @param MethodList            $paymentModel - helps load all payment instances
     */
    public function __construct(
        StoreManagerInterface $storeManager,
        CartInterface $quote,
        MethodList $paymentModel
    ) {
        $this->storeManager = $storeManager;
        $this->quote        = $quote;
        $this->paymentModel = $paymentModel;
    }

    /**
     * Retrieves all available payment methods and formats
     * them to Shopgate Merchant API ready format
     *
     * @return array() - e.g. ['id' => 'checkmo', 'title' => 'Money Order', 'is_active' => 1]
     */
    public function getPaymentMethods()
    {
        $this->quote->setStoreId($this->storeManager->getStore()->getId());

        $export         = [];
        $paymentMethods = $this->paymentModel->getAvailableMethods($this->quote);

        foreach ($paymentMethods as $method) {
            $export[] = [
                'id'        => $method->getCode(),
                'title'     => $method->getTitle(),
                'is_active' => 1
            ];
        }

        return $export;
    }
}
