<?php

/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */
namespace Shopgate\Base\Helper\Settings;

use Magento\Framework\Api\SimpleDataObjectConverter;

class Retriever
{

    /** @var array - allowed getSettings to export passed in DI */
    private $exportParams;
    /** @var Customer\Retriever */
    private $customerRetriever;
    /** @var Country\Retriever */
    private $countryRetriever;
    /** @var Payment\Retriever */
    private $paymentRetriever;
    /** @var Tax\Retriever */
    private $taxRetriever;

    /**
     * @param array              $exportParams
     * @param Customer\Retriever $customerGroupRetriever
     * @param Country\Retriever  $countryRetriever
     * @param Payment\Retriever  $paymentRetriever
     * @param Tax\Retriever      $taxRetriever
     *
     * @codeCoverageIgnore
     */
    public function __construct(
        Customer\Retriever $customerGroupRetriever,
        Country\Retriever $countryRetriever,
        Payment\Retriever $paymentRetriever,
        Tax\Retriever $taxRetriever,
        $exportParams = []
    ) {
        $this->exportParams      = $exportParams;
        $this->customerRetriever = $customerGroupRetriever;
        $this->countryRetriever  = $countryRetriever;
        $this->paymentRetriever  = $paymentRetriever;
        $this->taxRetriever      = $taxRetriever;
    }

    /**
     * Returns an array of certain settings of the shop
     *
     * @see http://developer.shopgate.com/plugin_api/system_information/get_settings
     *
     * @throws \ShopgateLibraryException on invalid log in data or hard errors like database failure.
     */
    public function getSettings()
    {
        return $this->methodLoader($this->getExportParams());
    }

    /**
     * Traverses method array and calls the
     * functions of this class
     *
     * @param array $methods - array(snake_case)
     *
     * @return array
     */
    private function methodLoader($methods)
    {
        foreach ($methods as $key => $param) {
            if (is_array($param)) {
                $methods[$key] = $this->methodLoader($this->getExportParams($key));
                continue;
            }
            $method = 'get' . SimpleDataObjectConverter::snakeCaseToUpperCamelCase($param);

            if (method_exists($this, $method)) {
                $methods[$param] = $this->{$method}();
                unset($methods[$key]);
            }
        }

        return $methods;
    }

    /**
     * Retrieves all parameters if no key is specified
     *
     * @param string|null $key
     *
     * @return string|array
     */
    private function getExportParams($key = null)
    {
        if ($key && isset($this->exportParams[$key])) {
            return $this->exportParams[$key];
        }

        return $this->exportParams;
    }

    /**
     * @codeCoverageIgnore
     * @return array
     */
    protected function getCustomerGroups()
    {
        return $this->customerRetriever->getCustomerGroups();
    }

    /**
     * @codeCoverageIgnore
     * @return array
     */
    protected function getAllowedAddressCountries()
    {
        return $this->countryRetriever->getAllowedAddressCountries();
    }

    /**
     * @codeCoverageIgnore
     * @return array
     */
    protected function getAllowedShippingCountries()
    {
        return $this->countryRetriever->getAllowedShippingCountries();
    }

    /**
     * @codeCoverageIgnore
     * @return array
     */
    protected function getProductTaxClasses()
    {
        return $this->taxRetriever->getProductTaxClasses();
    }

    /**
     * @codeCoverageIgnore
     * @return array
     */
    protected function getCustomerTaxClasses()
    {
        return $this->taxRetriever->getCustomerTaxClasses();
    }

    /**
     * @codeCoverageIgnore
     * @return array
     */
    protected function getTaxRates()
    {
        return $this->taxRetriever->getTaxRates();
    }

    /**
     * @codeCoverageIgnore
     * @return array
     */
    protected function getTaxRules()
    {
        return $this->taxRetriever->getTaxRules();
    }

    /**
     * @codeCoverageIgnore
     * @return array
     */
    protected function getPaymentMethods()
    {
        return $this->paymentRetriever->getPaymentMethods();
    }
}
