<?php
/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

namespace Shopgate\Base\Helper\Shipping;

use Magento\Store\Model\StoreManagerInterface;
use Magento\Directory\Model\ResourceModel\Country;
use Magento\Shipping\Model\Config;

class Main
{
    /**
     * @param StoreManagerInterface $storeManager
     * @param Country\Collection    $countryCollection
     * @param Config                $carrierConfig
     */
    public function __construct(
        StoreManagerInterface $storeManager,
        Country\Collection $countryCollection,
        Config $carrierConfig
    ) {
        $this->storeManager      = $storeManager;
        $this->countryCollection = $countryCollection;
        $this->carrierConfig     = $carrierConfig;
    }

    /**
     * Retrieves magento's allowed shipping countries
     *
     * @param null|string|int $storeId - store ID of which to get the ship countries
     *
     * @return mixed
     */
    public function getMageShippingCountries($storeId = null)
    {
        return $this
            ->countryCollection
            ->loadByStore($this->storeManager->getStore($storeId))
            ->getColumnValues('country_id');
    }

    /**
     * Retrieves all carriers that are currently
     * set to active in magento
     *
     * @param null|string|int $storeId - store ID of which to get the carriers
     *
     * @return mixed
     */
    public function getActiveCarriers($storeId = null)
    {
        return $this->carrierConfig->getActiveCarriers($this->storeManager->getStore($storeId));
    }
}
