<?php

/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

namespace Shopgate\Base\Helper\Shopgate;

use Shopgate\Base\Helper\Regions;

/**
 * Helps with data translation from shopgate customer data to mage data
 */
class Customer
{
    /** @var Regions */
    private $regionsHelper;

    /**
     * @param Regions $regionsHelper
     */
    public function __construct(Regions $regionsHelper)
    {
        $this->regionsHelper = $regionsHelper;
    }

    /**
     * @param \ShopgateCartBase | \ShopgateCustomer $order
     * @param \ShopgateAddress                      $address
     * @param bool                                  $saveInAddressBook
     *
     * @return array
     */
    public function createAddressData($order, \ShopgateAddress $address, $saveInAddressBook = false)
    {
        $phoneNumber = $order->getMobile() ? : $order->getPhone();
        $region      = $this->regionsHelper->getMageRegionByAddress($address);
        $street2     = $address->getStreet2() ? "\n" . $address->getStreet2() : '';
        $addressData = [
            'prefix'    => __($address->getGender() === 'f' ? 'Mrs.' : 'Mr.')->render(),
            'company'   => $address->getCompany(),
            'firstname' => $address->getFirstName(),
            'lastname'  => $address->getLastName(),
            'street'    => $address->getStreet1() . $street2,
            'city'      => $address->getCity(),
            'postcode'  => $address->getZipcode(),
            'telephone' => $phoneNumber ? : 'n.a.',
            'email'     => $order->getMail(),
            'country_id'           => $address->getCountry(),
            'region_id'            => $region->getId(),
            'save_in_address_book' => $saveInAddressBook,
        ];

        $addressData = array_merge($addressData, $this->getCustomFields($address));

        return $addressData;
    }

    /**
     * Creates a custom field array
     *
     * @param \ShopgateAddress | \ShopgateCustomer $object
     * todo-sg: refactor this method into Shopgate\Extended\Customer own class
     *
*@return array
     */
    public function getCustomFields($object)
    {
        $customFields = [];
        foreach ($object->getCustomFields() as $field) {
            $customFields[$field->getInternalFieldName()] = $field->getValue();
        }

        return $customFields;
    }
}
