<?php

/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */
namespace Shopgate\Base\Model\Carrier;

use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Quote\Model\Quote\Address\RateRequest;
use Magento\Quote\Model\Quote\Address\RateResult\ErrorFactory;
use Magento\Quote\Model\Quote\Address\RateResult\MethodFactory;
use Magento\Shipping\Model\Carrier\AbstractCarrier;
use Magento\Shipping\Model\Carrier\CarrierInterface;
use Magento\Shipping\Model\Rate\ResultFactory;
use Magento\Tax\Model\Config as TaxConfig;
use Psr\Log\LoggerInterface;
use Shopgate\Base\Model\Config as MainConfig;
use Shopgate\Base\Model\Shopgate\Extended\Base;

class Fix extends AbstractCarrier implements CarrierInterface
{
    /** @codingStandardsIgnoreStart */
    /** @var string */
    protected $_code = 'shopgate';
    /** @var bool */
    protected $_isFixed = true;
    /** /** @codingStandardsIgnoreEnd */
    /** @var string */
    protected $method = 'fix';
    /** @var ResultFactory */
    private $rateResultFactory;
    /** @var MethodFactory */
    private $rateMethodFactory;
    /** @var TaxConfig */
    private $taxConfig;
    /** @var MainConfig */
    private $mainConfig;
    /** @var Base */
    private $sgOrder;

    /**
     * @param ScopeConfigInterface $scopeConfig
     * @param ErrorFactory         $rateErrorFactory
     * @param LoggerInterface      $logger
     * @param ResultFactory        $rateResultFactory
     * @param MethodFactory        $rateMethodFactory
     * @param TaxConfig            $taxConfig
     * @param MainConfig           $mainConfig
     * @param Base                 $sgOrder
     * @param array                $data
     */
    public function __construct(
        ScopeConfigInterface $scopeConfig,
        ErrorFactory $rateErrorFactory,
        LoggerInterface $logger,
        ResultFactory $rateResultFactory,
        MethodFactory $rateMethodFactory,
        TaxConfig $taxConfig,
        MainConfig $mainConfig,
        Base $sgOrder,
        array $data = []
    ) {
        $this->rateResultFactory = $rateResultFactory;
        $this->rateMethodFactory = $rateMethodFactory;
        $this->taxConfig         = $taxConfig;
        $this->mainConfig        = $mainConfig;
        $this->sgOrder           = $sgOrder;
        parent::__construct($scopeConfig, $rateErrorFactory, $logger, $data);
    }

    /**
     * @return array
     */
    public function getAllowedMethods()
    {
        return [$this->getCarrierCode() => __($this->getConfigData('name'))->render()];
    }

    /**
     * @param RateRequest $request
     *
     * @return bool|\Magento\Shipping\Model\Rate\Result
     */
    public function collectRates(RateRequest $request)
    {
        if (!$this->sgOrder->getOrderNumber()) {
            return false;
        }

        /** @var \Magento\Shipping\Model\Rate\Result $result */
        $result = $this->rateResultFactory->create();

        /** @var \Magento\Quote\Model\Quote\Address\RateResult\Method $method */
        $method = $this->rateMethodFactory->create();
        $method->setData('carrier', $this->getCarrierCode());
        $method->setData('carrier_title', 'Shopgate');

        // TODO: fetch data from config or do fancy stuff and take from shipping_info of sgOrder
        $method->setData('method', $this->method);
        $method->setData('method_title', $this->getConfigData('name'));

        $storeViewId         = $this->mainConfig->getStoreViewId();
        $shippingIncludesTax = $this->taxConfig->shippingPriceIncludesTax($storeViewId);
        $shippingInfo        = $this->sgOrder->getShippingInfos();
        $shippingAmount      = $shippingIncludesTax ? $shippingInfo->getAmountGross() : $shippingInfo->getAmountNet();
        $shippingAmount      = round($shippingAmount, 2);

        $method->setPrice($shippingAmount);
        $method->setData('cost', $shippingAmount);

        $result->append($method);

        return $result;
    }
}
