<?php

/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

namespace Shopgate\Base\Model;

use Magento\Framework\App\ObjectManager;
use Magento\Store\Model\StoreManagerInterface;
use Shopgate\Base\Api\ExportInterface;
use Shopgate\Base\Api\ImportInterface;
use Shopgate\Base\Api\SettingsInterface;
use ShopgateCart;
use ShopgateCustomer;
use ShopgateOrder;

/**
 * Forwards Shopgate Merchant API requests to the right functions
 */
class Forwarder extends \ShopgatePlugin
{
    /** @var Config */
    protected $config;
    /** @var ExportInterface */
    protected $exportApi;
    /** @var StoreManagerInterface */
    private $storeManager;
    /** @var SettingsInterface */
    private $settingsApi;
    /** @var ImportInterface */
    private $importApi;

    /**
     * Gets called on initialization
     *
     * @codeCoverageIgnore
     */
    public function startup()
    {
        /** @var \Shopgate\Base\Helper\Initializer\Forwarder $forwarderInitializer */
        $manager              = ObjectManager::getInstance();
        $forwarderInitializer = $manager->get('Shopgate\Base\Helper\Initializer\Forwarder');
        $this->config         = $forwarderInitializer->getMainConfig();
        $this->settingsApi    = $forwarderInitializer->getSettingsInterface();
        $this->exportApi      = $forwarderInitializer->getExportInterface();
        $this->importApi      = $forwarderInitializer->getImportInterface();

        $configInitializer  = $forwarderInitializer->getConfigInitializer();
        $this->storeManager = $configInitializer->getStoreManager();
        $this->config->loadConfig();
    }

    /**
     * @inheritdoc
     */
    public function cron($jobname, $params, &$message, &$errorcount)
    {
        // TODO: Implement cron() method.
    }

    /**
     * @inheritdoc
     */
    public function getCustomer($user, $pass)
    {
        return $this->exportApi->getCustomerRaw($user, $pass);
    }

    /**
     * @inheritdoc
     */
    public function registerCustomer($user, $pass, ShopgateCustomer $customer)
    {
        $this->importApi->registerCustomerRaw($user, $pass, $customer);
    }

    /**
     * @inheritdoc
     */
    public function addOrder(ShopgateOrder $order)
    {
        return $this->importApi->addOrder($order);
    }

    /**
     * @inheritdoc
     */
    public function updateOrder(ShopgateOrder $order)
    {
        return $this->importApi->updateOrder($order);
    }

    /**
     * @inheritdoc
     */
    public function checkCart(ShopgateCart $cart)
    {
        return $this->exportApi->checkCartRaw($cart);
    }

    /**
     * @inheritdoc
     */
    public function checkStock(ShopgateCart $cart)
    {
        // TODO: Implement checkStock() method.
    }

    /**
     * @inheritdoc
     */
    public function getSettings()
    {
        return $this->settingsApi->getSettings(null, null, null);
    }

    /**
     * @inheritdoc
     */
    public function getOrders(
        $customerToken,
        $customerLanguage,
        $limit = 10,
        $offset = 0,
        $orderDateFrom = '',
        $sortOrder = 'created_desc'
    ) {
        // TODO: Implement getOrders() method.
    }

    /**
     * @inheritdoc
     */
    public function syncFavouriteList($customerToken, $items)
    {
        // TODO: Implement syncFavouriteList() method.
    }

    /**
     * @inheritdoc
     */
    protected function createMediaCsv()
    {
        // TODO: Implement createMediaCsv() method.
    }

    /**
     * @inheritdoc
     */
    protected function createItems($limit = null, $offset = null, array $uids = [])
    {
        if ($this->splittedExport) {
            $limit  = is_null($limit) ? $this->exportLimit : $limit;
            $offset = is_null($offset) ? $this->exportOffset : $offset;
        }

        $items = $this->exportApi->getItemsRaw($limit, $offset, $uids);

        foreach ($items as $item) {
            $this->addItemModel($item);
        }

        return $items;
    }

    /**
     * @inheritdoc
     */
    protected function createCategories($limit = null, $offset = null, array $uids = [])
    {
        if ($this->splittedExport) {
            $limit  = is_null($limit) ? $this->exportLimit : $limit;
            $offset = is_null($offset) ? $this->exportOffset : $offset;
        }

        $categories = $this->exportApi->getCategoriesRaw($limit, $offset, $uids);

        foreach ($categories as $category) {
            $this->addCategoryModel($category);
        }

        return $categories;
    }

    /**
     * @inheritdoc
     */
    protected function createReviews($limit = null, $offset = null, array $uids = [])
    {
        // TODO: Implement createReviews() method.
    }
}
