<?php

/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

namespace Shopgate\Base\Model\Redirect;

use Magento\Framework\HTTP\Header;
use Shopgate\Base\Model\Config as MainConfig;
use Shopgate\Base\Model\Utility\SgLoggerInterface;
use Shopgate_Helper_Redirect_Type_Http;
use Shopgate_Helper_Redirect_Type_Js;
use ShopgateBuilder;

/**
 * Class helps initialize the Shopgate builder
 * and assign tags to it
 */
class Builder
{

    /** @var MainConfig */
    private $config;
    /** @var Shopgate_Helper_Redirect_Type_Http */
    private $http;
    /** @var Shopgate_Helper_Redirect_Type_Js */
    private $js;
    /** @var Header */
    private $header;
    /** @var SgLoggerInterface */
    private $logger;

    /**
     * Builder constructor.
     *
     * @param MainConfig        $config
     * @param Header            $header
     * @param SgLoggerInterface $logger
     *
     * @codeCoverageIgnore
     */
    public function __construct(MainConfig $config, Header $header, SgLoggerInterface $logger)
    {
        $this->config = $config;
        $this->header = $header;
        $this->logger = $logger;
    }

    /**
     * Instantiates the HTTP redirect object
     *
     * @return Shopgate_Helper_Redirect_Type_Http | null
     */
    public function buildHttpRedirect()
    {
        if (!is_null($this->http)) {
            return $this->http;
        }

        $builder   = new ShopgateBuilder($this->initConfig());
        $userAgent = $this->header->getHttpUserAgent();

        try {
            $this->http = $builder->buildHttpRedirect($userAgent, $_GET, $_COOKIE);
        } catch (\ShopgateMerchantApiException $e) {
            $this->logger->error('HTTP > oAuth access token for store not set: ' . $e->getMessage());

            return null;
        } catch (\Exception $e) {
            $this->logger->error('HTTP > error in HTTP redirect: ' . $e->getMessage());

            return null;
        }

        return $this->http;
    }

    /**
     * @return MainConfig
     */
    private function initConfig()
    {
        $configs = $this->config->toArray();
        $this->config->loadArray($configs);
        $this->config->loadConfig();

        return $this->config;
    }

    /**
     * Instantiates the JS script builder object
     *
     * @return Shopgate_Helper_Redirect_Type_Js | null
     */
    public function buildJsRedirect()
    {
        if (!is_null($this->js)) {
            return $this->js;
        }

        $builder = new ShopgateBuilder($this->initConfig());

        try {
            $this->js = $builder->buildJsRedirect($_GET, $_COOKIE);
        } catch (\ShopgateMerchantApiException $e) {
            $this->logger->error('JS > oAuth access token for store not set: ' . $e->getMessage());

            return null;
        } catch (\Exception $e) {
            $this->logger->error('JS > error in HTTP redirect: ' . $e->getMessage());

            return null;
        }

        return $this->js;
    }
}
