<?php

/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

namespace Shopgate\Base\Model\Redirect;

use Magento\Framework\Event\ObserverInterface;
use Shopgate\Base\Helper\Redirect as RedirectHelper;
use Shopgate\Base\Model\Redirect\Type;
use Shopgate\Base\Model\Storage\Session;

class Observer implements ObserverInterface
{

    /** @var Type\Js */
    protected $jsRedirect;
    /** @var Type\Http */
    protected $httpRedirect;
    /** @var RedirectHelper */
    private $redirect;
    /** @var Session */
    private $session;

    /**
     * @param Type\Js        $js
     * @param Type\Http      $http
     * @param RedirectHelper $redirect
     * @param Session        $session
     */
    public function __construct(
        Type\Js $js,
        Type\Http $http,
        RedirectHelper $redirect,
        Session $session
    ) {
        $this->jsRedirect   = $js;
        $this->httpRedirect = $http;
        $this->redirect     = $redirect;
        $this->session      = $session;
    }

    /**
     * Runs the redirect list
     * todo-sg: may need to optimize this as this runs isAllowed() 2 times, different event handle or cache everything?
     *
     * @inheritdoc
     */
    public function execute(\Magento\Framework\Event\Observer $observer)
    {
        if (!$this->redirect->isAllowed()) {
            $this->session->unsetScript();

            return;
        }

        foreach ($this->getRedirects() as $redirect) {
            $redirect->run();
        }
    }

    /**
     * By default will run HTTP redirect with JS fallback if HTTP
     * does not work. If JS is set as the main redirect, it will
     * just run the JS redirect
     *
     * @return Type\TypeInterface[]
     */
    private function getRedirects()
    {
        $httpWithJsBackup = [
            $this->httpRedirect,
            $this->jsRedirect
        ];

        return $this->redirect->isTypeJavaScript() ? [$this->jsRedirect] : $httpWithJsBackup;
    }
}
