<?php

/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

namespace Shopgate\Base\Model\Redirect\Route\Tags;

use Magento\Store\Model\Store;
use Magento\Store\Model\StoreManagerInterface;
use Shopgate\Base\Api\Config\CoreInterface;
use Shopgate\Base\Api\Config\SgCoreInterface;
use Shopgate_Helper_Redirect_TagsGenerator;

class Generic
{
    /** @var StoreManagerInterface */
    protected $storeManager;
    /** @var CoreInterface */
    protected $config;

    /**
     * @param CoreInterface         $config
     * @param StoreManagerInterface $storeManager
     */
    public function __construct(CoreInterface $config, StoreManagerInterface $storeManager)
    {
        $this->config       = $config;
        $this->storeManager = $storeManager;
    }

    /**
     * Generates a default set of tags
     *
     * @param string $pageTitle
     *
     * @return array
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function generate($pageTitle)
    {
        return [
            Shopgate_Helper_Redirect_TagsGenerator::SITE_PARAMETER_SITENAME       => $this->getSiteName(),
            Shopgate_Helper_Redirect_TagsGenerator::SITE_PARAMETER_DESKTOP_URL    => $this->getShopUrl(),
            Shopgate_Helper_Redirect_TagsGenerator::SITE_PARAMETER_MOBILE_WEB_URL => $this->getMobileUrl(),
            Shopgate_Helper_Redirect_TagsGenerator::SITE_PARAMETER_TITLE          => $pageTitle
        ];
    }

    /**
     * @return string
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    private function getSiteName()
    {
        return $this->storeManager->getWebsite()->getName();
    }

    /**
     * @return string
     */
    private function getShopUrl()
    {
        return $this->config->getConfigByPath(Store::XML_PATH_UNSECURE_BASE_URL)->getValue();
    }

    /**
     * todo-sg: this might need a fallback of some sort as merchant may not have defined it
     *
     * @return string
     */
    private function getMobileUrl()
    {
        $cname = $this->config->getConfigByPath(SgCoreInterface::PATH_CNAME)->getValue();

        return rtrim($cname, '/');
    }
}
