<?php

/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */
namespace Shopgate\Base\Model\Redirect\Route\Type;

use Magento\Backend\App\Action\Context;
use Magento\Framework\App\Request\Http;
use Magento\Store\Model\StoreManagerInterface;
use Shopgate\Base\Model\Redirect\Route\Tags\Generic as GenericTags;
use Shopgate\Base\Model\Redirect\Route\TypeInterface;

class Generic implements TypeInterface
{
    const CONTROLLER_KEY = 'default';

    /** @var Context */
    protected $context;
    /** @var StoreManagerInterface */
    protected $storeManager;
    /** @var GenericTags */
    protected $tags;

    /**
     * @param Context               $context
     * @param StoreManagerInterface $storeManager
     * @param GenericTags           $tags
     */
    public function __construct(Context $context, StoreManagerInterface $storeManager, GenericTags $tags)
    {
        $this->context      = $context;
        $this->storeManager = $storeManager;
        $this->tags         = $tags;
    }

    /**
     * @inheritdoc
     */
    public function getCacheKey()
    {
        /** @var Http $request */
        $request = $this->context->getRequest();
        $storeId = $this->storeManager->getStore()->getId();

        return $storeId . '_' . $request->getRouteName() . '_' . $this::CONTROLLER_KEY . '_' . $this->getSpecialId();
    }

    /**
     * Returns the ID of the page, by default
     * there is no ID
     *
     * @return string
     */
    protected function getSpecialId()
    {
        return '';
    }

    /**
     * @inheritdoc
     */
    public function callScriptBuilder(\Shopgate_Helper_Redirect_Type_TypeInterface $redirect)
    {
        return $redirect->loadDefault();
    }

    /**
     * @inheritdoc
     */
    public function getTags()
    {
        return $this->tags->generate($this->getTitle());
    }

    /**
     * @inheritdoc
     */
    protected function getTitle()
    {
        return $this->storeManager->getWebsite()->getName();
    }
}
