<?php
/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

namespace Shopgate\Base\Model\Redirect\Route\Type;

use Magento\Backend\App\Action\Context;
use Magento\Cms\Model\PageFactory;
use Magento\Store\Model\StoreManagerInterface;
use Shopgate\Base\Api\Config\CoreInterface;
use Shopgate\Base\Api\Config\SgCoreInterface;
use Shopgate\Base\Model\Redirect\Route\Tags\Generic as GenericTags;
use Shopgate\Base\Model\Source\CmsMap;
use Zend_Serializer;

class Page extends Generic
{
    const CONTROLLER_KEY = 'page';

    /** @var PageFactory */
    private $pageFactory;
    /** @var CoreInterface */
    private $config;

    /**
     * @inheritdoc
     *
     * @param PageFactory   $pageFactory
     * @param CoreInterface $config
     */
    public function __construct(
        Context $context,
        StoreManagerInterface $storeManager,
        PageFactory $pageFactory,
        GenericTags $tags,
        CoreInterface $config
    ) {
        parent::__construct($context, $storeManager, $tags);
        $this->pageFactory = $pageFactory;
        $this->config      = $config;
    }

    /**
     * @inheritdoc
     */
    public function callScriptBuilder(\Shopgate_Helper_Redirect_Type_TypeInterface $redirect)
    {
        return $redirect->loadCms($this->getUrlKey());
    }

    /**
     * @return string
     * @throws \Zend_Serializer_Exception
     */
    private function getUrlKey()
    {
        $pageId       = $this->getSpecialId();
        $cmsMapConfig = $this->config->getConfigByPath(SgCoreInterface::PATH_CMS_MAP)->getValue();

        if (!empty($cmsMapConfig)) {
            $cmsMap = (array) Zend_Serializer::unserialize($cmsMapConfig);

            foreach ($cmsMap as $map) {
                if ($map[CmsMap::INPUT_ID_CMS_PAGE] === $pageId) {
                    return $map[CmsMap::INPUT_ID_URL_KEY];
                }
            }
        }

        return $this->getCurrentPage()->getIdentifier();
    }

    /**
     * Returns the ID of the page
     *
     * @inheritdoc
     */
    protected function getSpecialId()
    {
        return $this->context->getRequest()->getParam('page_id');
    }

    /**
     * @return \Magento\Cms\Model\Page
     */
    private function getCurrentPage()
    {
        return $this->pageFactory->create()->load($this->getSpecialId());
    }

    /**
     * Returns the page name
     *
     * @inheritdoc
     */
    protected function getTitle()
    {
        return $this->getCurrentPage()->getTitle();
    }
}
