<?php

/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

namespace Shopgate\Base\Model\Redirect\Type;

use Shopgate\Base\Model\Redirect\Builder;
use Shopgate\Base\Model\Redirect\Route\Utility;
use Shopgate\Base\Model\Storage\Cache;
use Shopgate\Base\Model\Storage\Session;
use Shopgate\Base\Model\Utility\SgLoggerInterface;

/**
 * Class responsible for the javascript HTML creation & saving to session
 * to be picked up by the frontend phtml template and output in the <head>
 */
class Js implements TypeInterface
{

    /** @var Session */
    private $session;
    /** @var Utility */
    private $routeUtility;
    /** @var Cache */
    private $cache;
    /** @var Builder */
    private $builder;
    /** @var SgLoggerInterface */
    private $logger;

    /**
     * @param Session           $session
     * @param Utility           $utility
     * @param Cache             $cache
     * @param Builder           $builder
     * @param SgLoggerInterface $logger
     *
     * @codeCoverageIgnore
     */
    public function __construct(
        Session $session,
        Utility $utility,
        Cache $cache,
        Builder $builder,
        SgLoggerInterface $logger
    ) {
        $this->session      = $session;
        $this->routeUtility = $utility;
        $this->cache        = $cache;
        $this->builder      = $builder;
        $this->logger       = $logger;
    }

    /**
     * @inheritdoc
     */
    public function run()
    {
        $route    = $this->routeUtility->getRoute();
        $cacheKey = $route->getCacheKey();
        $jsHeader = $this->cache->load($cacheKey);

        if ($jsHeader === false) {
            $redirect = $this->builder->buildJsRedirect();

            if (!$redirect) {
                return;
            }

            $redirect->getBuilder()->setSiteParameters($route->getTags());

            $jsHeader = $route->callScriptBuilder($redirect);
            $cached   = $this->cache->save($jsHeader, $cacheKey);

            if (!$cached) {
                $this->logger->debug(
                    'Could not save header to cache. It was enabled: ' . $this->cache->isEnabled() .
                    '. Header is empty: ' . empty($jsHeader)
                );
            }
        }

        $this->session->setScript($jsHeader);
    }
}
