<?php

/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

namespace Shopgate\Base\Model\Service\Config;

use Magento\Config\Model\ResourceModel\Config\Data\CollectionFactory;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\DataObject;
use Magento\Store\Model\ScopeInterface;
use Magento\Store\Model\StoreManagerInterface;
use Shopgate\Base\Api\Config\CoreInterface;

class Core implements CoreInterface
{
    /** @var CollectionFactory - core_config_data table collection */
    protected $configFactory;
    /** @var StoreManagerInterface */
    protected $storeManager;
    /** @var ScopeConfigInterface */
    protected $scopeConfig;

    /**
     * @param CollectionFactory     $configCollection
     * @param StoreManagerInterface $storeManager
     * @param ScopeConfigInterface  $scopeConfig
     *
     * @codeCoverageIgnore
     */
    public function __construct(
        CollectionFactory $configCollection,
        StoreManagerInterface $storeManager,
        ScopeConfigInterface $scopeConfig
    ) {
        $this->configFactory = $configCollection;
        $this->storeManager  = $storeManager;
        $this->scopeConfig   = $scopeConfig;
    }

    /**
     * @inheritdoc
     */
    public function getCollectionByValue($value)
    {
        return $this->configFactory->create()
                                   ->addFieldToSelect('*')
                                   ->addFieldToFilter('value', ['eq' => $value])
                                   ->addOrder('scope_id');
    }

    /**
     * @inheritdoc
     * @codeCoverageIgnore
     */
    public function getConfigByPath($path, $storeId = null)
    {
        $storeId = $storeId ? : $this->storeManager->getStore()->getId();
        $item    = $this->findConfigByStoreId($this->getCollectionByPath($path), $storeId);
        if (!$item->getValue()) {
            $type = $storeId ? ScopeInterface::SCOPE_STORES : ScopeConfigInterface::SCOPE_TYPE_DEFAULT;
            $item->setValue($this->scopeConfig->getValue($path, $type, $storeId));
        }

        return $item;
    }

    /**
     * @inheritdoc
     */
    public function findConfigByStoreId($collection, $storeId)
    {
        /** @var \Magento\Framework\App\Config\Value[] $storeConfigs */
        $storeConfigs = $collection->getItemsByColumnValue('scope', ScopeInterface::SCOPE_STORES);
        foreach ($storeConfigs as $store) {
            if ($store->getData('scope_id') == $storeId) {
                return $store;
            }
        }

        /** @var \Magento\Framework\App\Config\Value[] $websites */
        $websites  = $collection->getItemsByColumnValue('scope', ScopeInterface::SCOPE_WEBSITES);
        $websiteId = $this->storeManager->getStore($storeId)->getWebsiteId();
        foreach ($websites as $website) {
            if ($website->getData('scope_id') == $websiteId) {
                return $website;
            }
        }
        $item = $collection->getItemByColumnValue('scope', ScopeConfigInterface::SCOPE_TYPE_DEFAULT);

        return $item ? : new DataObject;
    }

    /**
     * @inheritdoc
     * @return \Magento\Config\Model\ResourceModel\Config\Data\Collection
     */
    public function getCollectionByPath($path)
    {
        return $this->configFactory->create()
                                   ->addFieldToSelect('*')
                                   ->addFieldToFilter('path', ['eq' => $path])
                                   ->addOrder('scope_id');
    }
}
