<?php

/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

namespace Shopgate\Base\Model\Service\Config;

use Magento\Config\Model\ResourceModel\Config\Data\Collection;
use Magento\Framework\App\Config\Value;
use Magento\Store\Model\ScopeInterface;
use Shopgate\Base\Api\Config\SgCoreInterface;
use ShopgateLibraryException;

class SgCore extends Core implements SgCoreInterface
{

    /**
     * @inheritdoc
     */
    public function getStoreId($shopNumber)
    {
        $configItem = $this->getShopNumberCollection($shopNumber)->getFirstItem();
        $scopeId    = $configItem->getData('scope_id');

        switch ($configItem->getData('scope')) {
            case ScopeInterface::SCOPE_STORES:
                return (int)$scopeId;
            case ScopeInterface::SCOPE_WEBSITES:
                /** @var \Magento\Store\Model\Website $website */
                $website = $this->storeManager->getWebsite($scopeId);

                return $website->getDefaultStore()->getStoreId();
            default:
                return $this->storeManager->getDefaultStoreView()->getId();
        }
    }

    /**
     * @inheritdoc
     * @return \Magento\Config\Model\ResourceModel\Config\Data\Collection
     */
    public function getShopNumberCollection($shopNumber)
    {
        return $this->configFactory->create()
                                   ->addFieldToFilter('path', self::PATH_SHOP_NUMBER)
                                   ->addValueFilter($shopNumber)
                                   ->setOrder('scope_id');
    }

    /**
     * @inheritdoc
     */
    public function getSaveScope($path, $shopNumber)
    {
        /**
         * Check if current store has value in provided path
         */
        $storeConfig = $this->getConfigByPath($path);
        if ($storeConfig->getData('value')) {
            return $storeConfig;
        }

        $shopConfig = $this->retrievePathScope($path, $shopNumber);

        return $shopConfig;
    }

    /**
     * Edge case checker, handles scenarios when multiple
     * scope id's have the same shop number.
     * Traverses the given collection of shopConfigs
     * to check whether a value in a given path exists.
     * If not, returns the first item of the collection.
     *
     * @param string $path
     * @param string $shopNumber
     *
     * @return Value | \Magento\Framework\DataObject
     * @throws ShopgateLibraryException
     */
    private function retrievePathScope($path, $shopNumber)
    {
        $shopConfigs = $this->getShopNumberCollection($shopNumber);

        if (!$shopConfigs->getSize()) {
            throw new ShopgateLibraryException(ShopgateLibraryException::PLUGIN_API_UNKNOWN_SHOP_NUMBER, false, true);
        }

        $firstItem = $shopConfigs->getFirstItem();

        /**
         * todo-sg: should be improved to use a single query, e.g. (scope = x AND scope_id = x) OR (scope = ...)
         *
         * @var Value $shopConfig
         */
        foreach ($shopConfigs as $shopConfig) {
            /** @var Collection $collection */
            $collection = $this
                ->getCollectionByPath($path)
                ->addFieldToFilter('scope', $shopConfig->getScope())
                ->addFieldToFilter('scope_id', $shopConfig->getScopeId());

            if ($collection->getItems()) {
                /** @var Value $propertyConfig */
                $propertyConfig = $collection->setOrder('scope_id')->getFirstItem();
                if ($propertyConfig->getData('value')) {
                    return $propertyConfig;
                }
            }
        }

        return $firstItem;
    }

    /**
     * @inheritdoc
     */
    public function isValid()
    {
        if (!empty($this->getConfigByPath(self::PATH_ACTIVE)->getValue())
            && !empty($this->getConfigByPath(self::PATH_API_KEY)->getValue())
            && !empty($this->getConfigByPath(self::PATH_SHOP_NUMBER)->getValue())
            && !empty($this->getConfigByPath(self::PATH_CUSTOMER_NUMBER)->getValue())
        ) {
            return true;
        }

        return false;
    }
}
