<?php

/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */
namespace Shopgate\Base\Model\Shopgate;

use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Model\AbstractModel;
use Shopgate\Base\Model\Resource\Shopgate\Order as OrderResource;

/**
 * @method int getShopgateOrderId()
 * @method Order setShopgateOrderId(int $id)
 * @method int getOrderId()
 * @method Order setOrderId(int $id)
 * @method int getStoreId()
 * @method Order setStoreId(int $id)
 * @method string getShopgateOrderNumber()
 * @method Order setShopgateOrderNumber(string $orderNumber)
 * @method int getIsShippingBlocked()
 * @method Order setIsShippingBlocked(int $flag)
 * @method int getIsPaid()
 * @method Order setIsPaid(int $flag)
 * @method int getIsSentToShopgate()
 * @method Order setIsSentToShopgate(int $flag)
 * @method int getIsCancellationSentToShopgate()
 * @method Order setIsCancellationSentToShopgate(int $flag)
 * @method int getIsTest()
 * @method Order setIsTest(int $flag)
 * @method int getIsCustomerInvoiceBlocked()
 * @method Order setIsCustomerInvoiceBlocked(int $flag)
 * @method string getReportedShippingCollections()
 * @method Order setReportedShippingCollections(string $text)
 * @method string getReceivedData()
 * @method Order setReceivedData(string $serializedData)
 * @method OrderResource _getResource()
 */
class Order extends AbstractModel
{
    /**
     * Define resource model
     *
     * @codingStandardsIgnoreStart
     */
    protected function _construct()
    {
        $this->_init('Shopgate\Base\Model\Resource\Shopgate\Order');
    }

    /**
     * @codingStandardsIgnoreEnd
     *
     * @param string $number
     *
     * @return Order
     * @throws LocalizedException
     */
    public function loadByShopgateOrderNumber($number)
    {
        $order = $this->_getResource()->getByOrderNumber($number);
        if (!empty($order)) {
            $this->setData($order);
        }

        return $this;
    }

    /**
     * @param string $id
     *
     * @return Order
     * @throws LocalizedException
     */
    public function loadByMageOrderId($id)
    {
        $order = $this->_getResource()->getByOrderId($id);
        if (!empty($order)) {
            $this->setData($order);
        }

        return $this;
    }

    /**
     * Rewriting default to avoid deprecation
     * and have the ability to modify the saving process later
     *
     * @throws \Exception
     */
    public function save()
    {
        $this->_getResource()->save($this);
    }
}
