<?php

/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

namespace Shopgate\Base\Model\Shopgate;

use Shopgate\Base\Api\OrderRepositoryInterface;
use Shopgate\Base\Model\Config;
use Shopgate\Base\Model\Shopgate\Extended\Base;

class OrderRepository implements OrderRepositoryInterface
{
    /** @var OrderFactory */
    private $orderFactory;
    /** @var Base */
    private $sgOrder;
    /** @var Config */
    private $config;

    /**
     * @param OrderFactory $orderFactory
     * @param Base         $sgOrder
     * @param Config       $config
     */
    public function __construct(OrderFactory $orderFactory, Base $sgOrder, Config $config)
    {
        $this->orderFactory = $orderFactory;
        $this->sgOrder      = $sgOrder;
        $this->config       = $config;
    }

    /**
     * @inheritdoc
     */
    public function getByMageOrder($id)
    {
        return $this->orderFactory->create()->loadByMageOrderId($id);
    }

    /**
     * @inheritdoc
     */
    public function createAndSave($mageOrderId)
    {
        $order = $this->orderFactory->create()
                                    ->setOrderId($mageOrderId)
                                    ->setStoreId($this->config->getStoreViewId())
                                    ->setShopgateOrderNumber($this->sgOrder->getOrderNumber())
                                    ->setIsShippingBlocked($this->sgOrder->getIsShippingBlocked())
                                    ->setIsPaid($this->sgOrder->getIsPaid())
                                    ->setIsTest($this->sgOrder->getIsTest())
                                    ->setIsCustomerInvoiceBlocked($this->sgOrder->getIsCustomerInvoiceBlocked());

        $order->setReceivedData(\Zend_Serializer::serialize($this->sgOrder->toArray()));
        $order->save();
    }

    /**
     * @inheritdoc
     */
    public function update(Order $order)
    {
        if ($this->sgOrder->getUpdatePayment()) {
            $order->setIsPaid($this->sgOrder->getIsPaid());
        }

        if ($this->sgOrder->getUpdateShipping()) {
            $order->setIsShippingBlocked($this->sgOrder->getIsShippingBlocked());
        }
    }

    /**
     * @inheritdoc
     */
    public function checkOrderExists($orderNumber, $throwExceptionOnDuplicate = false)
    {
        $sgOrder = $this->get($orderNumber);
        if ($throwExceptionOnDuplicate && $sgOrder->getId() !== null) {
            throw new \ShopgateLibraryException(
                \ShopgateLibraryException::PLUGIN_DUPLICATE_ORDER,
                $orderNumber,
                true
            );
        }

        return $sgOrder;
    }

    /**
     * @inheritdoc
     */
    public function get($number)
    {
        return $this->orderFactory->create()->loadByShopgateOrderNumber($number);
    }
}
