<?php

/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

namespace Shopgate\Base\Model\Source;

use Magento\Eav\Model\Entity\Attribute\Set;
use Magento\Eav\Model\Entity\AttributeFactory;
use Magento\Framework\Option\ArrayInterface;
use Shopgate\Base\Model\Storage\Session;
use Zend_Serializer;

class AttributeList implements ArrayInterface
{
    /** Cache key to use to save the list */
    const CACHE_KEY = 'attribute_list';

    /** @var AttributeFactory */
    private $attributeFactory;
    /** @var Session */
    private $session;

    /**
     * @param AttributeFactory $attributeFactory
     * @param Session          $session
     */
    public function __construct(AttributeFactory $attributeFactory, Session $session)
    {
        $this->attributeFactory = $attributeFactory;
        $this->session          = $session;
    }

    /**
     * @inheritdoc
     */
    public function toOptionArray()
    {
        $attributes = $this->session->getData(self::CACHE_KEY);
        if ($attributes) {
            return Zend_Serializer::unserialize($attributes);
        }

        $list = $this->getAttributeList();
        $this->session->setData(self::CACHE_KEY, Zend_Serializer::serialize($list));

        return $list;
    }

    /**
     * @return array - array(array('value' => 'Attribute Code', 'label' => 'Name of Attribute'), ...)
     */
    private function getAttributeList()
    {
        $collection = $this->attributeFactory->create()->getCollection();
        $collection->addFieldToFilter(Set::KEY_ENTITY_TYPE_ID, 4);
        $collection->getSelect()->where('frontend_label <> ?', null);
        $attributes = $collection->load()->getItems();

        $list[] = ['value' => 0, 'label' => 'Please Select'];
        foreach ($attributes as $attribute) {
            /** @var \Magento\Eav\Model\Entity\Attribute $attribute */
            $list[] = [
                'value' => $attribute->getAttributeCode(),
                'label' => $attribute->getDefaultFrontendLabel()
            ];
        }

        return $list;
    }
}
