<?php

/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

namespace Shopgate\Base\Model\Source;

use Magento\Config\Block\System\Config\Form\Field\FieldArray\AbstractFieldArray;
use Magento\Framework\DataObject;
use Shopgate\Base\Block\Adminhtml\Form\Field\CmsMap as CmsMapField;

class CmsMap extends AbstractFieldArray
{
    const INPUT_ID_CMS_PAGE = 'cms_page';
    const INPUT_ID_URL_KEY  = 'url_key';

    /**
     * @inheritdoc
     * @codingStandardsIgnoreStart
     */
    protected $_columns = [];
    /** @inheritdoc */
    protected $_addAfter = true;
    /** @inheritdoc */
    protected $_addButtonLabel;
    /** @var CmsMapField */
    protected $cmsPageRenderer;

    /**
     * @inheritdoc
     */
    public function renderCellTemplate($columnName)
    {
        if ($columnName === self::INPUT_ID_URL_KEY) {
            $this->_columns[$columnName]['class'] = 'input-text required-entry validate-identifier';
            $this->_columns[$columnName]['style'] = 'width:250px';
        }

        return parent::renderCellTemplate($columnName);
    }

    /**
     * @inheritdoc
     */
    protected function _prepareArrayRow(DataObject $row)
    {
        $options = [];
        $cmsPage = $row->getData('cms_page');
        if ($cmsPage) {
            $options['option_' . $this->getCmsPageRenderer()->calcOptionHash($cmsPage)] =
                'selected="selected"';
        }
        $row->setData('option_extra_attrs', $options);
    }

    /**
     * Returns a rendered for cms page mapping
     *
     * @codingStandardsIgnoreEnd
     * @return CmsMapField
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    protected function getCmsPageRenderer()
    {
        if (!$this->cmsPageRenderer) {
            $this->cmsPageRenderer = $this->getLayout()->createBlock(
                'Shopgate\Base\Block\Adminhtml\Form\Field\CmsMap',
                '',
                ['data' => ['is_render_to_js_template' => true]]
            );
        }

        return $this->cmsPageRenderer;
    }
    /** @codingStandardsIgnoreEnd */

    /**
     * @inheritdoc
     */
    protected function _prepareToRender()
    {
        $this->addColumn(
            self::INPUT_ID_CMS_PAGE,
            [
                'label'    => __('Page'),
                'renderer' => $this->getCmsPageRenderer(),
            ]
        );
        $this->addColumn(self::INPUT_ID_URL_KEY, ['label' => __('URL Key')]);
        $this->_addAfter = false;
    }
}
