<?php

/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */
namespace Shopgate\Base\Model\Source\CmsMap;

use Magento\Config\Model\Config\Backend\Serialized\ArraySerialized as MageArraySerialized;
use Magento\Framework\App\Cache\TypeListInterface;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\Data\Collection\AbstractDb;
use Magento\Framework\Message\ManagerInterface;
use Magento\Framework\Model\Context;
use Magento\Framework\Model\ResourceModel\AbstractResource;
use Magento\Framework\Registry;
use Shopgate\Base\Model\Source\CmsMap;

class ArraySerialized extends MageArraySerialized
{
    /** @var array */
    private $list;
    /** @var ManagerInterface */
    private $messageManager;

    /**
     * Sets our Shopgate validator
     *
     * @inheritdoc
     */
    public function __construct(
        Context $context,
        Registry $registry,
        ScopeConfigInterface $config,
        TypeListInterface $cacheTypeList,
        ManagerInterface $messageManager,
        AbstractResource $resource = null,
        AbstractDb $resourceCollection = null,
        array $data = []
    ) {
        $this->messageManager = $messageManager;

        return parent::__construct($context, $registry, $config, $cacheTypeList, $resource, $resourceCollection, $data);
    }

    /**
     * @inheritdoc
     */
    public function beforeSave()
    {
        $oldValues = $this->getValue();
        $newValues = $this->removeDuplicates($oldValues);

        if (count($oldValues) !== count($newValues)) {
            $this->setValue($newValues);
            $this->messageManager->addWarningMessage(
                __('Same page was assigned a URL Key multiple times! Duplicate entries were removed.')
            );
        }

        return parent::beforeSave();
    }

    /**
     * Searches for duplicate CMS pages before saving
     *
     * @param array $values
     *
     * @return array
     */
    public function removeDuplicates($values)
    {
        foreach ($values as $key => $option) {
            $value = $this->getCmsPageValue($option);
            if ($value) {
                if ($this->listHasValue($value)) {
                    unset($values[$key]);
                } else {
                    $this->setListValue($value);
                }
            }
        }

        return $values;
    }

    /**
     * @param string | array $option
     *
     * @return bool
     */
    private function getCmsPageValue($option)
    {
        if ($this->isOptionCmsPage($option)) {
            return $option[CmsMap::INPUT_ID_CMS_PAGE];
        }

        return false;
    }

    /**
     * @param string | array $option
     *
     * @return bool
     */
    private function isOptionCmsPage($option)
    {
        return is_array($option) && !empty($option[CmsMap::INPUT_ID_CMS_PAGE]);
    }

    /**
     * @param string $value
     *
     * @return bool
     */
    private function listHasValue($value)
    {
        return isset($this->list[$value]);
    }

    /**
     * Creates a list map of page keys
     * to figure out of there are duplicates
     * in it later
     *
     * @param string $value
     */
    private function setListValue($value)
    {
        $this->list[$value] = 1;
    }
}
