<?php

/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

namespace Shopgate\Base\Model\Utility;

use Shopgate\Base\Model\Utility\Profiler\SgProfile;

class SgProfiler
{
    /** @var SgLogger */
    protected $logger;
    /** @var array */
    protected $profiles = [];
    /** @var SgProfile */
    protected $currentProfile = false;

    /**
     * Loads only logger
     *
     * @param SgLogger $logger
     */
    public function __construct(SgLogger $logger)
    {
        $this->logger = $logger;
    }

    /**
     * Loads and starts a profile.
     * If nothing is provided, then
     * it starts a generic profile.
     *
     * @param string | null $identifier
     *
     * @return $this
     */
    public function start($identifier = null)
    {
        $this->loadProfile($identifier);
        $this->currentProfile->start();

        return $this;

    }

    /**
     * Retrieves the current profile
     *
     * @param string | null $identifier
     *
     * @return SgProfile
     */
    protected function loadProfile($identifier = null)
    {
        if (!$identifier) {
            $identifier = 'generic';
        }

        if (isset($this->profiles[$identifier])) {
            $this->currentProfile = $this->profiles[$identifier];
        } else {
            $this->currentProfile        = new SgProfile();
            $this->profiles[$identifier] = $this->currentProfile;
        }

        return $this->currentProfile;
    }

    /**
     * Ends timing on current profile
     *
     * @return $this
     */
    public function end()
    {
        $this->currentProfile->end();

        return $this;
    }

    /**
     * Retrieves the difference
     * between start and end time
     * of the profile in seconds
     *
     * @return float
     */
    public function getDuration()
    {
        return $this->currentProfile->getDuration();
    }

    /**
     * Prints a message to debug log
     *
     * @param string $message - message needs to contain "%s" to print timing
     *
     * @return bool
     */
    public function debug($message = "%s seconds")
    {
        $this->logger->debug(sprintf($message, $this->currentProfile->getDuration()));
    }
}
