<?php

/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */
namespace Shopgate\Base\Setup;

use Magento\Framework\DB\Ddl\Table;
use Magento\Framework\Setup\InstallSchemaInterface;
use Magento\Framework\Setup\ModuleContextInterface;
use Magento\Framework\Setup\SchemaSetupInterface;

class InstallSchema implements InstallSchemaInterface
{
    /**
     * Installs DB schema for a module
     *
     * @param SchemaSetupInterface   $setup
     * @param ModuleContextInterface $context
     *
     * @throws \Zend_Db_Exception
     */
    public function install(SchemaSetupInterface $setup, ModuleContextInterface $context)
    {
        $installer = $setup;

        $installer->startSetup();

        $table = $installer->getConnection()
            ->newTable($installer->getTable('shopgate_order'))
            ->addColumn(
                'shopgate_order_id',
                Table::TYPE_INTEGER,
                null,
                ['identity' => true, 'nullable' => false, 'primary' => true],
                'Entity ID'
            )->addColumn(
                'order_id',
                 Table::TYPE_INTEGER,
                 null,
                 ['unsigned' => true, 'nullable' => false, 'default' => '0'],
                 'Order Id'
            )->addColumn(
                'store_id',
                 Table::TYPE_SMALLINT,
                 null,
                 ['unsigned' => true],
                 'Store Id'
            )->addColumn(
                'shopgate_order_number',
                Table::TYPE_TEXT,
                20,
                ['nullable' => false]
            )->addColumn(
                'is_shipping_blocked',
                Table::TYPE_INTEGER,
                1,
                ['nullable' => false]
            )->addColumn(
                'is_paid',
                Table::TYPE_INTEGER,
                1,
                ['nullable' => false]
            )->addColumn(
                'is_sent_to_shopgate',
                Table::TYPE_INTEGER,
                1,
                ['nullable' => false]
            )->addColumn(
                'is_cancellation_sent_to_shopgate',
                Table::TYPE_INTEGER,
                1,
                ['nullable' => false]
            )->addColumn(
                'is_test',
                Table::TYPE_INTEGER,
                1,
                ['nullable' => false]
            )->addColumn(
                'is_customer_invoice_blocked',
                Table::TYPE_INTEGER,
                1,
                ['nullable' => false]
            )->addColumn(
                'reported_shipping_collections',
                Table::TYPE_TEXT
            )->addColumn(
                'received_data',
                Table::TYPE_TEXT
            )->addIndex(
                $installer->getIdxName('shopgate_order', ['order_id']),
                    ['order_id']
            )->addIndex(
                $installer->getIdxName('shopgate_order', ['store_id']),
                    ['store_id']
            )->addForeignKey(
                $installer->getFkName('shopgate_order', 'order_id', 'sales_order', 'entity_id'),
                'order_id',
                $installer->getTable('sales_order'),
                'entity_id',
                Table::ACTION_CASCADE
            )->addForeignKey(
                $installer->getFkName('shopgate_order', 'store_id', 'store', 'store_id'),
                'store_id',
                $installer->getTable('store'),
                'store_id',
                Table::ACTION_SET_NULL
            )->setComment('Shopgate Orders');

        $installer->getConnection()->createTable($table);

        $table = $installer->getConnection()
            ->newTable($installer->getTable('shopgate_customer'))
            ->addColumn(
                'id',
                Table::TYPE_INTEGER,
                null,
                ['identity' => true, 'nullable' => false, 'primary' => true]
            )->addColumn(
                'customer_id',
                Table::TYPE_INTEGER,
                null,
                ['unsigned' => true],
                'Customer ID'
            )->addColumn(
                'token',
                Table::TYPE_TEXT,
                255,
                ['nullable' => false]
            )->addIndex(
                $installer->getIdxName('shopgate_customer', ['customer_id']),
                ['customer_id']
            )->addForeignKey(
                $installer->getFkName('shopgate_customer', 'customer_id', 'customer_entity', 'entity_id'),
                'customer_id',
                $installer->getTable('customer_entity'),
                'entity_id',
                Table::ACTION_CASCADE
            )->setComment('Shopgate Customer');

        $installer->getConnection()->createTable($table);

        $installer->endSetup();
    }
}
