<?php

/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */
namespace Shopgate\Base\Test;

class Bootstrap extends \Magento\TestFramework\Helper\Bootstrap
{
    /** @var array - holds loaded Shopgate DI preferences */
    protected static $loaded = [];
    /** @var bool */
    protected static $libraryLoaded = false;

    /**
     * Object manager rewrite to include our DI's & Library
     *
     * @return \Magento\Framework\ObjectManagerInterface | \Magento\TestFramework\ObjectManager
     */
    public static function getObjectManager()
    {
        $objManager = parent::getObjectManager();

        self::initializeDi($objManager, 'Shopgate_Base');
        self::initializeDi($objManager, 'Shopgate_Export');
        //self::initializeDi($objManager, 'Shopgate_Import'); //todo-sg: need a way to pass it on per module basis
        self::loadShopgateLibrary($objManager);

        return $objManager;
    }

    /**
     * @param \Magento\Framework\ObjectManagerInterface $objManager
     * @param  string                                   $pluginName
     */
    public static function initializeDi($objManager, $pluginName)
    {
        if (!in_array($pluginName, self::$loaded)) {
            $reader      = $objManager->get('Magento\Framework\Module\Dir\Reader');
            $di          = $reader->getModuleDir('etc', $pluginName) . '/di.xml';
            $parser      = $objManager->get('Magento\Framework\Xml\Parser');
            $xml         = $parser->load($di)->xmlToArray();
            $preferences = [];

            foreach ($xml['config']['_value']['preference'] as $preference) {
                $preferences[$preference['_attribute']['for']] = $preference['_attribute']['type'];
            }
            $objManager->configure(['preferences' => $preferences]);
            self::$loaded[] = $pluginName;
        }
    }

    /**
     * @param \Magento\Framework\ObjectManagerInterface $objManager
     */
    public static function loadShopgateLibrary($objManager)
    {
        if (!self::$libraryLoaded) {
            /** @var \Magento\Framework\Module\Dir\Reader $reader */
            $reader  = $objManager->get('Magento\Framework\Module\Dir\Reader');
            $libPath = $reader->getModuleDir('', 'Shopgate_Base') . '/library/shopgate.php';

            if (file_exists($libPath)) {
                include_once $libPath;
            }
            self::$libraryLoaded = true;
        }
    }
}
