<?php
/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

namespace Shopgate\Base\Test\Integration;

use Magento\Store\Model\ScopeInterface;
use Shopgate\Base\Api\Config\SgCoreInterface;
use Shopgate\Base\Test\Integration\Db\ConfigManager;
use Shopgate\Base\Test\Integration\Db\WebsiteManager;

class Db extends \PHPUnit_Framework_TestCase
{

    /**
     * @var WebsiteManager
     */
    private $websiteConfig;

    /**
     * @var ConfigManager
     */
    private $config;

    /**
     * Init website manager
     */
    public function setUp()
    {
        $this->config        = new ConfigManager;
        $this->websiteConfig = new WebsiteManager;
    }

    /**
     * Test config manager and website manager
     *
     * @uses   \Shopgate\Base\Test\Integration\Db\ConfigManager::setConfigValue
     * @uses   \Shopgate\Base\Test\Integration\Db\ConfigManager::getConfigValue
     *
     * @covers \Shopgate\Base\Test\Integration\Db\WebsiteManager::createSite
     */
    public function testConfigUtility()
    {
        $site = $this->websiteConfig->createSite();
        $this->config->setConfigValue(
            sgCoreInterface::PATH_SHOP_NUMBER, '1234', ScopeInterface::SCOPE_WEBSITES, $site->getWebsite()->getId()
        );
        $val = $this->config->getConfigValue(
            sgCoreInterface::PATH_SHOP_NUMBER, ScopeInterface::SCOPE_WEBSITES, $site->getWebsite()->getId()
        );
        $this->assertEquals('1234', $val);
    }

    /**
     * Tests creation of a store using a created website as base
     *
     * @uses   \Shopgate\Base\Test\Integration\Db\WebsiteManager::createSite
     *
     * @covers \Shopgate\Base\Test\Integration\Db\WebsiteManager::createStore
     */
    public function testCreateAnotherStoreInWebsite()
    {
        $site  = $this->websiteConfig->createSite();
        $site2 = $this->websiteConfig->createStore($site);

        $this->assertEquals($site->getWebsite()->getId(), $site2->getWebsite()->getId());
        $this->assertNotEquals($site->getStore()->getId(), $site2->getStore()->getId());
    }

    /**
     * Tests config setting & getting on store level
     *
     * @covers   \Shopgate\Base\Test\Integration\Db\ConfigManager::setConfigValue
     * @covers   \Shopgate\Base\Test\Integration\Db\ConfigManager::getConfigValue
     */
    public function testStoreConfigSetting()
    {
        $site = $this->websiteConfig->createSite();

        $this->config->setConfigValue(
            sgCoreInterface::PATH_SHOP_NUMBER, '1234', ScopeInterface::SCOPE_STORES, $site->getStore()->getId()
        );
        $val = $this->config->getConfigValue(
            sgCoreInterface::PATH_SHOP_NUMBER, ScopeInterface::SCOPE_STORES, $site->getStore()->getId()
        );
        $this->assertEquals('1234', $val);
    }

    /**
     * Remove all created websites after each test
     *
     * @after
     */
    public function cleanup()
    {
        $this->websiteConfig->removeSites();
        $this->config->removeConfigs();
    }
}
