<?php
/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

namespace Shopgate\Base\Test\Integration\Db;

use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\TestFramework\Helper\Bootstrap;

class ConfigManager
{
    /**
     * Holds config values to cleanup
     *
     * @var array
     */
    protected $cleanup = [];

    /**
     * Retrieve configuration node value
     *
     * @param string      $configPath
     * @param string      $scopeType
     * @param string|null $scopeCode
     *
     * @return string
     */
    public function getConfigValue(
        $configPath,
        $scopeType = ScopeConfigInterface::SCOPE_TYPE_DEFAULT,
        $scopeCode = null
    ) {
        $objectManager = Bootstrap::getObjectManager();
        $result        = null;
        if ($scopeCode !== false) {
            /** @var ScopeConfigInterface $scopeConfig */
            $scopeConfig = $objectManager->get('Magento\Framework\App\Config\ScopeConfigInterface');
            $result      = $scopeConfig->getValue(
                $configPath,
                $scopeType,
                $scopeCode
            );
        }

        return $result;
    }

    /**
     * Assign configuration node value
     *
     * @param string      $configPath
     * @param string      $value
     * @param string      $scopeType
     * @param string|null $scopeId
     */
    public function setConfigValue(
        $configPath,
        $value,
        $scopeType = ScopeConfigInterface::SCOPE_TYPE_DEFAULT,
        $scopeId = null
    ) {
        $objectManager = Bootstrap::getObjectManager();
        /** @var \Magento\Framework\App\Config\ConfigResource\ConfigInterface $scopeConfig */
        $scopeConfig = $objectManager->get(
            '\Magento\Framework\App\Config\ConfigResource\ConfigInterface'
        );
        if (empty($scopeId)) {
            $scopeId = 0;
            if (strpos($configPath, 'default/') === 0) {
                $configPath = substr($configPath, 8);
            }
            $scopeConfig->saveConfig(
                $configPath,
                $value,
                $scopeType,
                $scopeId
            );
        } else {
            $scopeConfig->saveConfig(
                $configPath,
                $value,
                $scopeType,
                $scopeId
            );
        }

        $this->cleanup[] = [$configPath, $scopeType, $scopeId];
    }

    /**
     * Cleans up all the deleted config params
     */
    public function removeConfigs()
    {
        /** @var \Magento\Framework\App\Config\ConfigResource\ConfigInterface $configResource */
        $configResource = Bootstrap::getObjectManager()->get(
            '\Magento\Framework\App\Config\ConfigResource\ConfigInterface'
        );
        foreach ($this->cleanup as list($path, $scopeType, $scopeId)) {
            $configResource->deleteConfig($path, $scopeType, $scopeId);
        }
    }
}
