<?php
/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

namespace Shopgate\Base\Test\Integration\Db\Data;

use Magento\Store\Api\Data\GroupInterface;
use Magento\Store\Api\Data\StoreInterface;
use Magento\Store\Api\Data\WebsiteInterface;
use Magento\TestFramework\Helper\Bootstrap;

class Website
{
    /** @var WebsiteInterface */
    private $website;
    /** @var  GroupInterface */
    private $group;
    /** @var StoreInterface */
    private $store;

    /**
     * @param string $websiteCode
     *
     * @return WebsiteInterface
     */
    public function createWebsite($websiteCode = null)
    {
        if (!$websiteCode) {
            $websiteCode = 'website_' . rand(0, 99999);
        }

        return $this->website = Bootstrap::getObjectManager()
                                         ->create('Magento\Store\Api\Data\WebsiteInterface')
                                         ->setCode($websiteCode)
                                         ->setName('WebsiteName')
                                         ->save();
    }

    /**
     * @param int         $websiteId
     * @param int         $rootCatId
     * @param string|null $groupName
     *
     * @return GroupInterface
     */
    public function createGroup($websiteId = null, $rootCatId = 0, $groupName = null)
    {
        if (!$groupName) {
            $groupName = 'group_' . rand(0, 99999);
        }
        if (!$websiteId) {
            $websiteId = $this->getWebsite()->getId();
        }

        return $this->group = Bootstrap::getObjectManager()
                                       ->create('Magento\Store\Api\Data\GroupInterface')
                                       ->setWebsiteId($websiteId)
                                       ->setName($groupName)
                                       ->setRootCategoryId($rootCatId)
                                       ->save();

    }

    /**
     * @return WebsiteInterface
     */
    public function getWebsite()
    {
        return $this->website;
    }

    /**
     * @param WebsiteInterface $website
     */
    public function setWebsite($website)
    {
        $this->website = $website;
    }

    /**
     * @param int         $websiteId
     * @param int         $storeGroupId
     * @param string|null $storeCode
     * @param string|null $storeName
     * @param int         $isActive
     *
     * @return StoreInterface
     */
    public function createStore(
        $websiteId = null,
        $storeGroupId = null,
        $storeCode = null,
        $storeName = null,
        $isActive = 1
    ) {
        $id = rand(0, 99999);

        if (!$storeName) {
            $storeName = 'Store ' . $id;
        }

        if (!$storeCode) {
            $storeCode = 'store_' . $id;
        }

        if (!$websiteId) {
            $websiteId = $this->getWebsite()->getId();
        }

        if (!$storeGroupId) {
            $storeGroupId = $this->getGroup()->getId();
        }

        return $this->store = Bootstrap::getObjectManager()
                                       ->create('Magento\Store\Api\Data\StoreInterface')
                                       ->setCode($storeCode)
                                       ->setWebsiteId($websiteId)
                                       ->setGroupId($storeGroupId)
                                       ->setName($storeName)
                                       ->setIsActive($isActive)
                                       ->save();
    }

    /**
     * @return GroupInterface
     */
    public function getGroup()
    {
        return $this->group;
    }

    /**
     * @param GroupInterface $group
     */
    public function setGroup($group)
    {
        $this->group = $group;
    }

    /**
     * Removes Website/Group/Store from DB
     *
     * @return $this
     */
    public function delete()
    {
        /** @var \Magento\Store\Model\ResourceModel\Store\Interceptor $resource */
        $resource = $this->getStore()->getResource();
        $resource->getConnection()->delete(
            $resource->getMainTable(),
            [$resource->getIdFieldName() . ' = ' . $this->getStore()->getId()]
        );
        /** @var \Magento\Store\Model\ResourceModel\Group\Interceptor $resource */
        $resource = $this->getGroup()->getResource();
        $resource->getConnection()->delete(
            $resource->getMainTable(),
            [$resource->getIdFieldName() . ' = ' . $this->getGroup()->getId()]
        );
        /** @var \Magento\Store\Model\ResourceModel\Website\Interceptor $resource */
        $resource = $this->getWebsite()->getResource();
        $resource->getConnection()->delete(
            $resource->getMainTable(),
            [$resource->getIdFieldName() . ' = ' . $this->getWebsite()->getId()]
        );

        unset($this->website);
        unset($this->store);
        unset($this->group);

        return $this;
    }

    /**
     * @return StoreInterface
     */
    public function getStore()
    {
        return $this->store;
    }
}
