<?php
/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

namespace Shopgate\Base\Test\Integration\Db;

class WebsiteManager
{

    /**
     * @var Data\Website[]
     */
    protected $websites = [];

    /**
     * The magic unicorns create a store for you
     *
     * @return Data\Website
     */
    public function createSite()
    {
        $site = new Data\Website();
        $site->createWebsite();
        $site->createGroup();
        $site->createStore();
        $this->addWebsite($site);

        return $site;
    }

    /**
     * Adds the website to the stack for removal
     * purposes later
     *
     * @param Data\Website $website
     */
    private function addWebsite(Data\Website $website)
    {
        $this->websites[] = $website;
    }

    /**
     * Creates a new store from the website object
     *
     * @param Data\Website $site
     *
     * @return Data\Website
     */
    public function createStore(Data\Website $site)
    {
        $website = new Data\Website();
        $website->setWebsite($site->getWebsite());
        $website->setGroup($site->getGroup());
        $website->createStore($site->getWebsite()->getId(), $site->getGroup()->getId());
        $this->addWebsite($website);

        return $website;
    }

    /**
     * Remove all created sites
     *
     * todo-sg: clean DB indexes
     */
    public function removeSites()
    {
        foreach ($this->getWebsites() as $website) {
            $website->delete();
        }

        $this->websites = [];
    }

    /**
     * @return Data\Website[]
     */
    public function getWebsites()
    {
        return $this->websites;
    }

    /**
     * Gives us item in first position
     *
     * @return Data\Website
     */
    public function getFirstItem()
    {
        return current($this->websites);
    }
}
