<?php
/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

namespace Shopgate\Base\Test\Integration\Model\Redirect\Route\Type;

use Magento\Framework\App\Request\Http;
use Shopgate\Base\Api\Config\SgCoreInterface;
use Shopgate\Base\Model\Redirect\Route\Type;
use Shopgate\Base\Test\Bootstrap;
use Shopgate\Base\Test\Integration\Db\ConfigManager;

/**
 * @coversDefaultClass \Shopgate\Base\Model\Redirect\Type\Js
 * @group Shopgate_Base_Redirect
 */
class JsTest extends \PHPUnit_Framework_TestCase
{
    /** @var \Shopgate\Base\Model\Redirect\Type\Js */
    private $class;
    /** @var ConfigManager */
    private $cfg;
    /** @var \Magento\Backend\App\Action\Context */
    private $context;

    public function setUp()
    {
        $manager       = Bootstrap::getObjectManager();
        $this->cfg     = new ConfigManager();
        $this->class   = $manager->create('Shopgate\Base\Model\Redirect\Type\Js');
        $this->context = $manager->create('Magento\Backend\App\Action\Context');
    }

    /**
     * @covers ::run
     */
    public function testInit()
    {
        $this->setUpConfig();

        /** @var Http $req */
        $req = $this->context->getRequest();
        $req->setControllerName(Type\Page::CONTROLLER_KEY)
            ->setParam('page_id', '5');

        $this->class->run();

        /** @var \Shopgate\Base\Model\Storage\Session $session */
        $session = Bootstrap::getObjectManager()->get('Shopgate\Base\Model\Storage\Session');
        $script  = $session->getScript();
        $this->assertNotEmpty($script);
    }

    /**
     * Helps set up the main config to redirect shop
     */
    private function setUpConfig()
    {
        $this->cfg->setConfigValue(SgCoreInterface::PATH_ACTIVE, 1);
        $this->cfg->setConfigValue(SgCoreInterface::PATH_SHOP_NUMBER, 1234);
        $this->cfg->setConfigValue(SgCoreInterface::PATH_CUSTOMER_NUMBER, 2345);
        $this->cfg->setConfigValue(SgCoreInterface::PATH_API_KEY, 1111111111);
        $this->cfg->setConfigValue(SgCoreInterface::PATH_CNAME, 'lao-cha.com');
    }

    /**
     * @after
     */
    public function tearDown()
    {
        $this->cfg->removeConfigs();

        /** @var \Shopgate\Base\Model\Storage\Session $session */
        $session = Bootstrap::getObjectManager()->get('Shopgate\Base\Model\Storage\Session');
        $session->unsetScript();
    }
}
