<?php
/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

namespace Shopgate\Base\Test\Integration\Model\Redirect\Route;

use Magento\Framework\App\Request\Http;
use Shopgate\Base\Model\Redirect\Route\Type;
use Shopgate\Base\Test\Bootstrap;

/**
 * @coversDefaultClass \Shopgate\Base\Model\Redirect\Route\Utility
 * @group Shopgate_Base_Redirect
 */
class UtilityTest extends \PHPUnit_Framework_TestCase
{
    /** @var \Shopgate\Base\Model\Redirect\Route\Utility */
    private $class;
    /** @var \Magento\Backend\App\Action\Context */
    private $context;

    public function setUp()
    {
        $manager       = Bootstrap::getObjectManager();
        $this->class   = $manager->create('Shopgate\Base\Model\Redirect\Route\Utility');
        $this->context = $manager->create('Magento\Backend\App\Action\Context');
    }

    /**
     * @param string $controller
     * @param string $expectedType
     *
     * @covers ::getRoute
     * @dataProvider routeProvider
     */
    public function testGetRouteDefault($controller, $expectedType)
    {
        /** @var Http $req */
        $req = $this->context->getRequest();
        $req->setControllerName($controller);
        $route = $this->class->getRoute();

        $this->assertInstanceOf('Shopgate\Base\Model\Redirect\Route\Type\\' . $expectedType, $route);
    }

    /**
     * Provides all the routes
     *
     * @return array
     */
    public function routeProvider()
    {
        return [
            'Undefined type'     => ['la-la-land', 'Generic'],
            'Category page type' => [Type\Category::CONTROLLER_KEY, 'Category'],
            'Home page type'     => [Type\Home::CONTROLLER_KEY, 'Home'],
            'CMS page type'      => [Type\Page::CONTROLLER_KEY, 'Page'],
            'Product page type'  => [Type\Product::CONTROLLER_KEY, 'Product'],
            'Search page type'   => [Type\Search::CONTROLLER_KEY, 'Search']
        ];
    }
}
