<?php
/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

namespace Shopgate\Base\Test\Integration\Model\Service\Config;

use Magento\Store\Model\ScopeInterface;
use Magento\TestFramework\Helper\Bootstrap;
use Shopgate\Base\Api\Config\SgCoreInterface;
use Shopgate\Base\Model\Service\Config\SgCore;
use Shopgate\Base\Test\Integration\Db\ConfigManager;
use Shopgate\Base\Test\Integration\Db\WebsiteManager;

/**
 * @coversDefaultClass Shopgate\Base\Model\Service\Config\Core
 */
class CoreTest extends \PHPUnit_Framework_TestCase
{
    /** @var WebsiteManager */
    private $siteManager;
    /** @var ConfigManager */
    private $cfgManager;
    /** @var SgCore */
    private $method;

    /**
     * Set up essentials
     */
    public function setUp()
    {
        $this->cfgManager  = new ConfigManager;
        $this->siteManager = new WebsiteManager;
        $this->method      = Bootstrap::getObjectManager()->create('Shopgate\Base\Model\Service\Config\Core');
    }

    /**
     * @uses \Shopgate\Base\Model\Service\Config\Core::getConfigByPath
     * @covers ::getCollectionByPath
     * @covers ::findConfigByStoreId
     */
    public function testFindConfigByStoreId()
    {
        $this->cfgManager->setConfigValue(SgCoreInterface::PATH_SHOP_NUMBER, '1235');
        $this->cfgManager->setConfigValue(SgCoreInterface::PATH_SHOP_NUMBER, '1236', ScopeInterface::SCOPE_STORES, 1);

        $collection = $this->method->getCollectionByPath(SgCoreInterface::PATH_SHOP_NUMBER);
        $this->assertCount(2, $collection);

        /**
         * Checks store specific config retrieval
         */
        $config = $this->method->findConfigByStoreId($collection, 1);
        $this->assertEquals('1236', $config->getData('value'));

        /**
         * Checks default path config retrieval
         */
        $defaultConfig = $this->method->findConfigByStoreId($collection, null);
        $this->assertEquals('1235', $defaultConfig->getData('value'));

        /**
         * Checks store config, then loads website config
         */
        $site = $this->siteManager->createSite();
        $this->cfgManager->setConfigValue(
            SgCoreInterface::PATH_SHOP_NUMBER,
            '1237',
            ScopeInterface::SCOPE_WEBSITES,
            $site->getWebsite()->getId()
        );
        $collection2 = $this->method->getCollectionByPath(SgCoreInterface::PATH_SHOP_NUMBER);
        $websiteCfg  = $this->method->findConfigByStoreId($collection2, $site->getStore()->getId());
        $this->assertEquals('1237', $websiteCfg->getData('value'));

        /**
         * Checks empty object on empty collection
         */
        $emptyConfig = $this->method->getConfigByPath('invalid_path', 1);
        $this->assertEmpty($emptyConfig->getData('value'));
    }

    /**
     * @after
     */
    public function cleanup()
    {
        $this->cfgManager->removeConfigs();
        $this->siteManager->removeSites();
    }
}
