<?php
/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

namespace Shopgate\Base\Test\Unit\Helper\Settings;

use Magento\Framework\TestFramework\Unit\Helper\ObjectManager;

/**
 * @coversDefaultClass \Shopgate\Base\Helper\Settings\Retriever
 */
class RetrieverTest extends \PHPUnit_Framework_TestCase
{

    /**
     * @var ObjectManager
     */
    private $objectManager;

    /**
     * Load object manager for initialization
     */
    public function setUp()
    {
        $this->objectManager = new ObjectManager($this);
    }

    /**
     * @param $expected - expected array to come out
     * @param $params   - params to pass into the retriever's constructor, impersonating DI
     *
     * @covers ::getSettings
     * @covers ::methodLoader
     *
     * @dataProvider settingsProvider
     */
    public function testGetSettings($expected, $params)
    {
        /** @var \Shopgate\Base\Helper\Settings\Retriever $retrieverModel */
        $retrieverModel = $this->objectManager->getObject(
            '\Shopgate\Base\Helper\Settings\Retriever',
            [
                'exportParams' => $params
            ]
        );

        $actual = $retrieverModel->getSettings();

        $this->assertEquals($expected, $actual);
    }

    /**
     * @covers ::getExportParams
     */
    public function testGetExportParams()
    {
        $params = $this->settingsProvider();
        /** @var \Shopgate\Base\Helper\Settings\Retriever $retrieverModel */
        $retrieverModel = $this->objectManager->getObject(
            '\Shopgate\Base\Helper\Settings\Retriever',
            [
                'exportParams' => $params[0]['params']
            ]
        );

        $reflection = new \ReflectionClass($retrieverModel);
        $method     = $reflection->getMethod('getExportParams');
        $method->setAccessible(true);

        $result = $method->invoke($retrieverModel);
        $this->assertCount(3, $result);

        $result2 = $method->invoke($retrieverModel, 'tax');
        $this->assertCount(2, $result2);
    }

    /**
     * @return array
     */
    public function settingsProvider()
    {
        return
            [
                [
                    'expected' => [
                        'tax'             => [
                            'tax_rates' => null,
                            'tax_rules' => null
                        ],
                        'customer_groups' => null,
                        'payment_methods' => null,
                    ],
                    'params'   => [
                        'customer_groups',
                        'payment_methods',
                        'tax' => [
                            'tax_rates',
                            'tax_rules'
                        ]
                    ]
                ]
            ];
    }
}
