<?php
/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

namespace Shopgate\Base\Test\Unit\Helper\Settings\Tax;

use Magento\Framework\TestFramework\Unit\Helper\ObjectManager;
use Magento\Tax\Model\Calculation\Rate;
use Magento\Tax\Model\ResourceModel\Calculation\Rate\Collection;
use Shopgate\Base\Helper\Settings\Tax\Retriever;


/**
 * @coversDefaultClass \Shopgate\Base\Helper\Settings\Tax\Retriever
 */
class RetrieverTest extends \PHPUnit_Framework_TestCase
{

    /**
     * @var ObjectManager
     */
    private $objectManager;

    /**
     * Load object manager for initialization
     */
    public function setUp()
    {
        $this->objectManager = new ObjectManager($this);
    }

    /**
     * @param array $rates
     *
     * @return \PHPUnit_Framework_MockObject_MockObject
     */
    private function getTaxCollectionStubWithRates($rates)
    {
        $taxRates = $this->getMockBuilder(Collection::class)
                         ->disableOriginalConstructor()
                         ->getMock();

        $taxRates->method('getIterator')
                 ->will($this->returnValue(new \ArrayIterator($rates)));

        return $taxRates;
    }

    /**
     * @return \PHPUnit_Framework_MockObject_MockObject
     */
    private function getBlankTaxRateDouble()
    {
        $taxRate = $this->getMockBuilder(Rate::class)
                        ->disableOriginalConstructor()
                        ->getMock();

        return $taxRate;
    }

    /**
     * @param string $expectedCodeType
     * @param string $expectedZipCodePattern
     * @param string $zipCodeTo
     * @param string $zipCodeFrom
     * @param string $zipIsRange
     * @param string $postCode
     *
     * @covers ::getTaxRates
     * @dataProvider taxRateProvider
     */
    public function testGetTaxRatesReturns(
        $expectedCodeType,
        $expectedZipCodePattern,
        $zipCodeTo,
        $zipCodeFrom,
        $zipIsRange,
        $postCode
    ) {
        $taxRate = $this->getBlankTaxRateDouble();

        $taxRate->method('getZipIsRange')
                ->will($this->returnValue($zipIsRange));

        $taxRate->method('getZipFrom')
                ->will($this->returnValue($zipCodeFrom));

        $taxRate->method('getZipTo')
                ->will($this->returnValue($zipCodeTo));

        $taxRate->method('getTaxPostcode')
                ->will($this->returnValue($postCode));

        $taxRates = $this->getTaxCollectionStubWithRates([$taxRate]);

        /** @var \Shopgate\Base\Helper\Settings\Tax\Retriever $retrieverModel */
        $retrieverModel = $this->objectManager->getObject(
            '\Shopgate\Base\Helper\Settings\Tax\Retriever',
            [
                'taxRates' => $taxRates
            ]
        );

        $actual = $retrieverModel->getTaxRates();

        $this->assertEquals($actual[0]['zipcode_type'], $expectedCodeType);
        $this->assertEquals($actual[0]['zipcode_pattern'], $expectedZipCodePattern);
        $this->assertEquals($actual[0]['zipcode_range_from'], $zipCodeFrom);
        $this->assertEquals($actual[0]['zipcode_range_to'], $zipCodeTo);
    }

    /**
     * @return array
     */
    public function taxRateProvider()
    {
        return [
            'taxRate with range'    => [Retriever::ZIP_CODE_TYPE_RANGE, '', '1', '2', '3', ''],
            'taxRate without range' => [Retriever::ZIP_CODE_TYPE_ALL, '', '', '', '', ''],
            'taxRate with code'     => [Retriever::ZIP_CODE_TYPE_PATTERN, '23', '', '', '', '23']
        ];
    }
}
