<?php
/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

namespace Shopgate\Base\Test\Unit\Model;

use Magento\Framework\TestFramework\Unit\Helper\ObjectManager;

/**
 * @coversDefaultClass \Shopgate\Base\Model\Config
 */
class ConfigTest extends \PHPUnit_Framework_TestCase
{
    /** @var ObjectManager */
    private $objectManager;
    /** @var \Shopgate\Base\Model\Config */
    private $configModel;

    /**
     * Load object manager for initialization
     */
    public function setUp()
    {
        $this->objectManager = new ObjectManager($this);
        $this->configModel   = $this->getMock('Shopgate\Base\Model\Config', [], [], '', false);
    }

    /**
     * Test property retriever from doc block
     *
     * @param $expected - expected from provider
     * @param $property - property name of ShopgateConfig class
     *
     * @covers       ::getPropertyType()
     * @dataProvider propertyProvider
     */
    public function testGetProperty($expected, $property)
    {
        $reflection = new \ReflectionClass($this->configModel);
        $method     = $reflection->getMethod('getPropertyType');
        $method->setAccessible(true);
        $result = $method->invoke($this->configModel, $property);

        $this->assertEquals($expected, $result);
    }

    /**
     * Test casting of value based on property
     *
     * @param $expected
     * @param $value
     * @param $property
     *
     * @covers       ::castToType()
     * @dataProvider castTypeProvider
     */
    public function testCastToType($expected, $value, $property)
    {
        $reflection = new \ReflectionClass($this->configModel);
        $method     = $reflection->getMethod('castToType');
        $method->setAccessible(true);
        $result = $method->invoke($this->configModel, $value, $property);

        $this->assertEquals($expected, $result);
    }

    /**
     * Test type conversion to save to core_config table
     *
     * @param $expected
     * @param $property
     * @param $value
     *
     * @covers ::prepareForDatabase()
     * @dataProvider prepareForDatabaseProvider
     */
    public function testPrepareForDatabase($expected, $value, $property)
    {
        $reflection = new \ReflectionClass($this->configModel);
        $method     = $reflection->getMethod('prepareForDatabase');
        $method->setAccessible(true);
        $result = $method->invoke($this->configModel, $property, $value);

        $this->assertEquals($expected, $result);
    }

    /**
     * @return array
     */
    public function propertyProvider()
    {
        return [
            ['string', 'apikey'],
            ['int', 'shop_number'],
            ['bool', 'shop_is_active'],
            ['array', 'api_urls'],
            ['string', 'random_var'],
        ];
    }

    /**
     * @return array
     */
    public function castTypeProvider()
    {
        return [
            'int converts to str'    => ['1', 1, 'apikey'],
            'str converts to str'    => ['test', 'test', 'apikey'],
            'str converts to int'    => [345, '345', 'shop_number'],
            'int converts to int'    => [345, 345, 'shop_number'],
            'int converts to bool'   => [true, 1, 'shop_is_active'],
            'bool converts to bool'  => [true, true, 'shop_is_active'],
            'str converts to array'  => [[1, 2, 3], '1,2,3', 'api_urls'],
            'str converts to array2' => [['test1', 'test2'], 'test1,test2', 'api_urls'],
        ];
    }

    /**
     * @return array
     */
    public function prepareForDatabaseProvider()
    {
        return [
            'array converts to str'  => ['test1,test2', ['test1', 'test2'], 'api_urls'],
            'array converts to str2' => ['1,2,3', [1, 2, 3], 'api_urls'],
            'bool converts to int'   => [1, true, 'shop_is_active'],
            'str converts to int'    => ['345', '345', 'shop_number'],
        ];
    }
}
