<?php

/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

namespace Shopgate\Export\Helper;

use Magento\Framework\Api\SimpleDataObjectConverter;
use Shopgate\Base\Model\Config;
use Shopgate\Base\Model\Utility\SgLoggerInterface;
use Shopgate\Export\Helper\Quote as QuoteHelper;

class Cart
{
    /** @var Config */
    private $config;
    /** @var SgLoggerInterface */
    private $logger;
    /** @var QuoteHelper */
    private $quoteHelper;
    /** @var array */
    private $quoteMethods;

    /**
     * @param Config            $config
     * @param SgLoggerInterface $logger
     * @param QuoteHelper       $quoteHelper
     * @param array             $quoteMethods
     */
    public function __construct(
        Config $config,
        SgLoggerInterface $logger,
        QuoteHelper $quoteHelper,
        array $quoteMethods = []
    ) {
        $this->config       = $config;
        $this->logger       = $logger;
        $this->quoteHelper  = $quoteHelper;
        $this->quoteMethods = $quoteMethods;
    }

    /**
     * Takes in the allowed cart methods loaded up in the Base's DI
     *
     * @return array
     * @throws \Exception
     */
    public function loadSupportedMethods()
    {
        $methods = [];
        $this->quoteHelper->load($this->quoteMethods);

        foreach ($this->config->getSupportedFieldsCheckCart() as $rawMethod) {
            $method = 'get' . SimpleDataObjectConverter::snakeCaseToUpperCamelCase($rawMethod);
            $this->logger->debug('Starting method ' . $method);
            $methods[$rawMethod] = $this->{$method}();
            $this->logger->debug('Finished method ' . $method);
        }
        $this->quoteHelper->cleanup();

        return $methods;
    }

    /**
     * @return \ShopgateShippingMethod[]
     */
    protected function getShippingMethods()
    {
        //todo-sg: finish method implementation
    }

    /**
     * @return string
     */
    protected function getCurrency()
    {
        //todo-sg: get currency of current shop, can also implement in the main Config as it falls back to that
    }

    /**
     * @return \ShopgateCartCustomer
     */
    protected function getCustomer()
    {
        //todo-sg: get customer data, seems like we can reuse Helper/Customer.php here quite a bit
    }

    /**
     * @return \ShopgateCartItem[]
     * @throws \Exception
     */
    protected function getItems()
    {
        return $this->quoteHelper->getValidatedItems();
    }

    /**
     * @return \ShopgateExternalCoupon[]
     */
    protected function getExternalCoupons()
    {
        return $this->quoteHelper->getValidatedCoupons();
    }

    /**
     * @return \ShopgatePaymentMethod[]
     */
    protected function getPaymentMethods()
    {
        //todo-sg: return payment methods available for this cart
    }
}
