<?php

/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

namespace Shopgate\Export\Helper\Category;

use Magento\Store\Model\StoreManagerInterface;
use Shopgate\Base\Model\Utility\SgLoggerInterface;
use Shopgate\Export\Model\Export\Category;

class Retriever
{
    /** @var SgLoggerInterface */
    private $log;
    /** @var StoreManagerInterface */
    private $storeManager;
    /** @var Utility */
    private $utility;

    /**
     * @param SgLoggerInterface     $logger
     * @param StoreManagerInterface $storeManager
     * @param Utility               $utility
     *
     * @codeCoverageIgnore
     */
    public function __construct(
        SgLoggerInterface $logger,
        StoreManagerInterface $storeManager,
        Utility $utility
    ) {
        $this->log          = $logger;
        $this->storeManager = $storeManager;
        $this->utility      = $utility;
    }

    /**
     * @see \Shopgate\Export\Api\ExportInterface::getCategories
     *
     * @param null | string $limit
     * @param null | string $offset
     * @param array         $uids
     *
     * @return Category[]
     */
    public function getCategories($limit = null, $offset = null, array $uids = [])
    {
        $this->log->access('Start Category Export...');
        $this->log->debug('Start Category Export...');

        $rootCatId = $this->storeManager->getGroup()->getRootCategoryId();

        $this->log->debug('Root-Category-Id: ' . $rootCatId);
        $this->log->debug('Start Category-Tree Build ...');

        $export = $this->utility->buildCategoryTree($rootCatId, $uids, $offset, $limit);

        $this->log->debug('End Category-Tree Build...');
        $this->log->access('Finished Category Export...');
        $this->log->debug('Finished Category Export...');

        return $export;
    }
}
