<?php

/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

namespace Shopgate\Export\Helper\Category;

use Magento\Catalog\Model\CategoryFactory;
use Shopgate\Base\Model\Utility\SgLoggerInterface;
use Shopgate\Export\Model\Export\Category;
use Shopgate\Export\Model\Export\CategoryFactory as ExportFactory;

class Utility
{
    /** @var SgLoggerInterface */
    private $log;
    /** @var CategoryFactory */
    private $categoryFactory;
    /** @var ExportFactory */
    private $exportFactory;

    /**
     * @param CategoryFactory   $categoryFactory
     * @param ExportFactory     $exportFactory
     * @param SgLoggerInterface $logger
     */
    public function __construct(
        CategoryFactory $categoryFactory,
        ExportFactory $exportFactory,
        SgLoggerInterface $logger
    ) {
        $this->categoryFactory = $categoryFactory;
        $this->exportFactory   = $exportFactory;
        $this->log             = $logger;
    }

    /**
     * Traverses category tree and exports a set of Shopgate Categories
     *
     * @param int         $parentId - root category to use
     * @param null|array  $uIds     - UID's to export
     * @param null|string $offset   - offset position to export from
     * @param null|string $limit    - limit of products to export
     *
     * @return Category[]
     */
    public function buildCategoryTree($parentId, $uIds, $offset, $limit)
    {
        $this->log->debug('Build Tree with Parent-ID: ' . $parentId);

        if (empty($uIds)) {
            $category = $this->categoryFactory->create()->load($parentId);
            /** @var \Magento\Catalog\Model\ResourceModel\Category\Tree $tree */
            /** @noinspection PhpParamsInspection */
            $rootNode = $category->getTreeModel()->load()->getNodeById($parentId);
            $uIds     = $category->getTreeModel()->getChildren($rootNode);
        }

        if (!is_null($offset) && !is_null($offset)) {
            $uIds = array_slice($uIds, $offset, $limit);
        }

        $export      = [];
        $maxPosition = $this->getMaximumCategoryPosition();

        foreach ($uIds as $categoryId) {
            $this->log->debug('Load Category with ID: ' . $categoryId);
            /** @var \Magento\Catalog\Model\Category $category */
            $category            = $this->categoryFactory->create()->load($categoryId);
            $categoryExportModel = $this->exportFactory->create();
            $categoryExportModel->setItem($category);
            $categoryExportModel->setParentId($parentId);
            $categoryExportModel->setMaximumPosition($maxPosition);
            $export[] = $categoryExportModel->generateData();
        }

        return $export;
    }

    /**
     * Retrieves the maximum category number + 100
     *
     * @return int
     */
    private function getMaximumCategoryPosition()
    {
        $maxCategoryPosition = $this->categoryFactory->create()
                                                     ->getCollection()
                                                     ->setOrder('position', 'DESC')
                                                     ->getFirstItem()
                                                     ->getData('position');
        $this->log->debug('Max Category Position: ' . $maxCategoryPosition);

        return $maxCategoryPosition + 100;
    }
}
