<?php

/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

namespace Shopgate\Export\Helper\Customer;

use Magento\Customer\Model\AccountManagement;
use Magento\Framework\Exception\EmailNotConfirmedException;
use Magento\Framework\Exception\InvalidEmailOrPasswordException;
use Shopgate\Base\Helper\Customer\Utility;
use Shopgate\Base\Model\Utility\SgLoggerInterface;
use ShopgateLibraryException;

class Retriever
{
    /** @var AccountManagement */
    protected $accountManagement;
    /** @var SgLoggerInterface */
    private $log;
    /** @var Utility */
    private $utility;

    /**
     * @param AccountManagement $accountManagement
     * @param SgLoggerInterface $sgLoggerInterface
     * @param Utility           $utility
     */
    public function __construct(
        AccountManagement $accountManagement,
        SgLoggerInterface $sgLoggerInterface,
        Utility $utility
    ) {
        $this->accountManagement = $accountManagement;
        $this->log               = $sgLoggerInterface;
        $this->utility           = $utility;
    }

    /**
     * @param string $user - The user name the customer entered at Shopgate Connect.
     * @param string $pass - The password the customer entered at Shopgate Connect.
     *
     * @return \ShopgateCustomer
     * @throws \ShopgateLibraryException
     */
    public function getCustomer($user, $pass)
    {
        try {
            $magentoCustomer = $this->accountManagement->authenticate($user, $pass);
        } catch (InvalidEmailOrPasswordException $e) {
            throw new ShopgateLibraryException(
                ShopgateLibraryException::PLUGIN_WRONG_USERNAME_OR_PASSWORD,
                null,
                false,
                false
            );
        } catch (EmailNotConfirmedException $e) {
            throw new ShopgateLibraryException(
                ShopgateLibraryException::PLUGIN_CUSTOMER_ACCOUNT_NOT_CONFIRMED,
                null,
                false,
                false
            );
        } catch (\Exception $e) {
            $this->log->error($e->getMessage());
            throw new ShopgateLibraryException(
                ShopgateLibraryException::PLUGIN_CUSTOMER_UNKNOWN_ERROR,
                null,
                false,
                false
            );
        }
        $shopgateCustomer = $this->utility->loadByMagentoCustomer($magentoCustomer);

        return $shopgateCustomer;
    }
}
