<?php

/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

namespace Shopgate\Export\Helper\Product;

use Magento\Catalog\Api\Data\ProductCustomOptionValuesInterface;
use Magento\Catalog\Model\Product;
use Magento\Catalog\Model\Product as MageProduct;
use Magento\Catalog\Model\Product\Option;
use Magento\Catalog\Model\Product\Visibility;
use Magento\CatalogInventory\Model\ResourceModel\Stock\Item as StockItemResource;
use Magento\ConfigurableProduct\Model\Product\Type\Configurable;
use Magento\GroupedProduct\Model\Product\Type\Grouped;
use Magento\Tax\Model\Config as TaxConfig;
use Shopgate\Base\Helper\Product\Utility as BaseProductUtility;
use Shopgate\Export\Api\ExportInterface;
use Shopgate\Export\Model\Config\Source\Description;
use Shopgate_Model_Catalog_CategoryPath;

class Utility extends BaseProductUtility
{
    /** Identifier manufacturer */
    const DEFAULT_ATTRIBUTE_MANUFACTURER = 'manufacturer';
    /** Identifier price type percent */
    const DEFAULT_PRICE_TYPE_PERCENT = 'percent';
    /** Default description linebreak format */
    const DEFAULT_DESCRIPTION_LINEBREAK_PATTERN = '%s<br /><br />%s';

    /** @var array */
    protected $inputTypes = [
        'field'     => \Shopgate_Model_Catalog_Input::DEFAULT_INPUT_TYPE_TEXT,
        'area'      => \Shopgate_Model_Catalog_Input::DEFAULT_INPUT_TYPE_AREA,
        'select'    => \Shopgate_Model_Catalog_Input::DEFAULT_INPUT_TYPE_SELECT,
        'drop_down' => \Shopgate_Model_Catalog_Input::DEFAULT_INPUT_TYPE_SELECT,
        'radio'     => \Shopgate_Model_Catalog_Input::DEFAULT_INPUT_TYPE_SELECT,
        'checkbox'  => \Shopgate_Model_Catalog_Input::DEFAULT_INPUT_TYPE_SELECT,
        'multiple'  => \Shopgate_Model_Catalog_Input::DEFAULT_INPUT_TYPE_SELECT,
        'multi'     => \Shopgate_Model_Catalog_Input::DEFAULT_INPUT_TYPE_SELECT,
        'date'      => \Shopgate_Model_Catalog_Input::DEFAULT_INPUT_TYPE_DATE,
        'date_time' => \Shopgate_Model_Catalog_Input::DEFAULT_INPUT_TYPE_DATETIME,
        'time'      => \Shopgate_Model_Catalog_Input::DEFAULT_INPUT_TYPE_TIME
    ];
    /** @var array */
    protected $weightUnits = [
        'kgs'  => \Shopgate_Model_Catalog_Product::DEFAULT_WEIGHT_UNIT_KG,
        'g'    => \Shopgate_Model_Catalog_Product::DEFAULT_WEIGHT_UNIT_GRAM,
        'auto' => \Shopgate_Model_Catalog_Product::DEFAULT_WEIGHT_UNIT_DEFAULT,
        'lb'   => \Shopgate_Model_Catalog_Product::DEFAULT_WEIGHT_UNIT_POUND,
        'oz'   => \Shopgate_Model_Catalog_Product::DEFAULT_WEIGHT_UNIT_OUNCE
    ];

    /**
     * @param MageProduct $product
     *
     * @return bool
     */
    public function isVisibleInCategories($product)
    {
        $validVisibilities = [
            Visibility::VISIBILITY_BOTH,
            Visibility::VISIBILITY_IN_CATALOG
        ];

        return in_array($product->getVisibility(), $validVisibilities);
    }

    /**
     * @param MageProduct $product
     *
     * @return string
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function getManufacturer($product)
    {
        $manufacturer = $product->getManufacturer();

        if ($manufacturer) {
            $manufacturer =
                $product->getResource()
                        ->getAttribute(self::DEFAULT_ATTRIBUTE_MANUFACTURER)
                        ->getSource()
                        ->getOptionText($manufacturer);
        }

        return $manufacturer;
    }

    /**
     * @param MageProduct $product
     *
     * @return \Shopgate_Model_Catalog_Visibility
     */
    public function setVisibility($product)
    {
        $visibility = new \Shopgate_Model_Catalog_Visibility();
        switch ($product->getVisibility()) {
            case Visibility::VISIBILITY_BOTH:
                $level = \Shopgate_Model_Catalog_Visibility::DEFAULT_VISIBILITY_CATALOG_AND_SEARCH;
                break;
            case Visibility::VISIBILITY_IN_CATALOG:
                $level = \Shopgate_Model_Catalog_Visibility::DEFAULT_VISIBILITY_CATALOG;
                break;
            case Visibility::VISIBILITY_IN_SEARCH:
                $level = \Shopgate_Model_Catalog_Visibility::DEFAULT_VISIBILITY_SEARCH;
                break;
            case Visibility::VISIBILITY_NOT_VISIBLE:
                $level = \Shopgate_Model_Catalog_Visibility::DEFAULT_VISIBILITY_NOTHING;
                break;
            default:
                $level = null;
        }

        $visibility->setLevel($level);
        $visibility->setMarketplace(true);

        return $visibility;
    }

    /**
     * @param MageProduct $product
     *
     * @return \Magento\CatalogInventory\Api\Data\StockItemInterface
     */
    public function getStockItem($product)
    {
        $stockItem = $this->stockItemFactory->create();

        $this->stockItemResource->loadByProductId(
            $stockItem,
            $product->getId(),
            $this->storeManager->getWebsite()->getId()
        );

        return $stockItem;
    }

    /**
     * @param string $mageType
     *
     * @return mixed
     */
    public function mapInputType($mageType)
    {
        if (isset($this->inputTypes[$mageType])) {
            return $this->inputTypes[$mageType];
        }

        return false;
    }

    /**
     * @param $weightUnit
     *
     * @return string
     */
    public function getWeightUnit($weightUnit)
    {
        if (isset($this->weightUnits[$weightUnit])) {
            return $this->weightUnits[$weightUnit];
        }

        return $weightUnit;
    }

    /**
     * @param string $inputType
     * @param Option $option
     *
     * @return \Shopgate_Model_Catalog_Validation
     */
    public function buildInputValidation($inputType, $option)
    {
        $validation = new \Shopgate_Model_Catalog_Validation();

        switch ($inputType) {
            case \Shopgate_Model_Catalog_Input::DEFAULT_INPUT_TYPE_TEXT:
            case \Shopgate_Model_Catalog_Input::DEFAULT_INPUT_TYPE_AREA:
                $validationType = \Shopgate_Model_Catalog_Validation::DEFAULT_VALIDATION_VARIABLE_STRING;
                $validation->setValue($option->getMaxCharacters());
                break;
            case \Shopgate_Model_Catalog_Input::DEFAULT_INPUT_TYPE_FILE:
                $validationType = \Shopgate_Model_Catalog_Validation::DEFAULT_VALIDATION_TYPE_FILE;
                $validation->setValue($option->getFileExtension());
                break;
            case \Shopgate_Model_Catalog_Input::DEFAULT_INPUT_TYPE_DATE:
                $validationType = \Shopgate_Model_Catalog_Validation::DEFAULT_VALIDATION_VARIABLE_DATE;
                break;
            case \Shopgate_Model_Catalog_Input::DEFAULT_INPUT_TYPE_DATETIME:
                $validationType = \Shopgate_Model_Catalog_Validation::DEFAULT_VALIDATION_VARIABLE_DATE;
                break;
            case \Shopgate_Model_Catalog_Input::DEFAULT_INPUT_TYPE_TIME:
                $validationType = \Shopgate_Model_Catalog_Validation::DEFAULT_VALIDATION_VARIABLE_TIME;
                break;
            case \Shopgate_Model_Catalog_Input::DEFAULT_INPUT_TYPE_SELECT:
            case \Shopgate_Model_Catalog_Input::DEFAULT_INPUT_TYPE_RADIO:
            case \Shopgate_Model_Catalog_Input::DEFAULT_INPUT_TYPE_CHECKBOX:
            case \Shopgate_Model_Catalog_Input::DEFAULT_INPUT_TYPE_MULTIPLE:
                $validationType = \Shopgate_Model_Catalog_Validation::DEFAULT_VALIDATION_VARIABLE_STRING;
                break;
            default:
                return $validation;
        }

        $validation->setValidationType($validationType);

        return $validation;
    }

    /**
     * @param Option      $option
     * @param MageProduct $item
     *
     * @return mixed
     */
    public function getInputValuePrice($option, $item)
    {
        return $this->getOptionValuePrice($option, $item);
    }

    /**
     * @param Option | ProductCustomOptionValuesInterface $option
     * @param MageProduct                                 $item
     *
     * @return mixed
     */
    public function getOptionValuePrice($option, $item)
    {
        if ($option->getPriceType() == self::DEFAULT_PRICE_TYPE_PERCENT) {
            return $item->getFinalPrice() * ($option->getPrice() / 100);
        }

        return $option->getPrice();
    }

    /**
     * @param array  $relationIds
     * @param string $type
     *
     * @return \Shopgate_Model_Catalog_Relation
     */
    public function createRelationProducts(array $relationIds, $type)
    {
        $relationModel = new \Shopgate_Model_Catalog_Relation();
        $relationModel->setType($type);
        $relationModel->setValues($relationIds);

        return $relationModel;
    }

    /**
     * @param Option      $option
     * @param MageProduct $item
     *
     * @return array
     */
    public function buildInputOptions($option, $item)
    {
        $optionValues = [];

        foreach ($option->getValues() as $id => $value) {
            $inputOption = new \Shopgate_Model_Catalog_Option();
            $inputOption->setUid($id);
            $inputOption->setLabel($value->getTitle());
            $inputOption->setSortOrder($value->getSortOrder());
            $inputOption->setAdditionalPrice($this->getOptionValuePrice($value, $item));
            $optionValues[] = $inputOption;
        }

        return $optionValues;
    }

    /**
     * @param MageProduct $item
     *
     * @return float
     */
    public function getTaxRate($item)
    {
        return $this->taxCalculation->getDefaultCalculatedRate($item->getTaxClassId());
    }

    /**
     * @param MageProduct $item
     * @param MageProduct $parentItem
     *
     * @return string
     */
    public function getDeepLinkUrl($item, $parentItem = null)
    {
        $deepLink = $parentItem && $item->getVisibility() == Visibility::VISIBILITY_NOT_VISIBLE
            ? $this->utility->parseUrl($parentItem->getProductUrl())
            : $this->utility->parseUrl($item->getProductUrl());

        return $deepLink;
    }

    /**
     * @return bool
     */
    public function priceIncludesTax()
    {
        return $this->taxConfig->priceIncludesTax($this->storeManager->getStore());
    }

    /**
     * Create a category export for a product
     *
     * @param string | int $categoryId
     * @param null         $position
     *
     * @return Shopgate_Model_Catalog_CategoryPath
     */
    public function getExportCategory($categoryId, $position = null)
    {
        $category = new Shopgate_Model_Catalog_CategoryPath();
        $category->setSortOrder($position);
        $category->setUid($categoryId);

        return $category;
    }

    /**
     * @param int | string $productId
     * @param int | string $categoryId
     *
     * @return int
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function getPositionInCategory($productId, $categoryId)
    {
        /** @var \Magento\Catalog\Model\Category $category */
        $category  = $this->categoryRepository->get($categoryId);
        $positions = $category->getProductsPosition();

        return (int) isset($positions[$productId]) ? $positions[$productId] : 1;
    }

    /**
     * Returns the description of current product as per
     * Shopgate Sys > Config instructions
     *
     * @param MageProduct $product
     *
     * @return string
     * @throws \Exception
     */
    public function getIndividualDescription(Product $product)
    {
        $descriptionConfig = $this->sgCore->getConfigByPath(ExportInterface::PATH_PROD_DESCRIPTION);

        switch ($descriptionConfig->getValue()) {
            case Description::ID_DESCRIPTION_AND_SHORT_DESCRIPTION:
                $description = sprintf(
                    self::DEFAULT_DESCRIPTION_LINEBREAK_PATTERN,
                    $product->getDescription(),
                    $product->getShortDescription()
                );
                break;
            case Description::ID_SHORT_DESCRIPTION_AND_DESCRIPTION:
                $description = sprintf(
                    self::DEFAULT_DESCRIPTION_LINEBREAK_PATTERN,
                    $product->getShortDescription(),
                    $product->getDescription()
                );
                break;
            case Description::ID_SHORT_DESCRIPTION:
                $description = $product->getShortDescription();
                break;
            default:
                $description = $product->getDescription();
        }

        return $this->filter->getPageFilter()->filter($description);
    }
}
