<?php

/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

namespace Shopgate\Export\Model\Service;

use Shopgate\Base\Model\Shopgate\Extended\Base;
use Shopgate\Export\Api\ExportInterface;
use Shopgate\Export\Helper\Cart;
use Shopgate\Export\Helper\Category\Retriever as CategoryRetriever;
use Shopgate\Export\Helper\Customer\Retriever as CustomerRetriever;
use Shopgate\Export\Helper\Product\Retriever as ProductRetriever;

class Export implements ExportInterface
{
    /** @var CustomerRetriever */
    protected $customerHelper;
    /** @var CategoryRetriever */
    private $categoryRetriever;
    /** @var ProductRetriever */
    private $productRetriever;
    /** @var Cart */
    private $cartHelper;
    /** @var Base */
    private $sgBase;

    /**
     * @param Cart              $cartHelper
     * @param CategoryRetriever $categoryRetriever
     * @param ProductRetriever  $productRetriever
     * @param CustomerRetriever $customerHelper
     * @param Base              $base
     *
     * @codeCoverageIgnore
     */
    public function __construct(
        Cart $cartHelper,
        CategoryRetriever $categoryRetriever,
        ProductRetriever $productRetriever,
        CustomerRetriever $customerHelper,
        Base $base
    ) {
        $this->cartHelper        = $cartHelper;
        $this->categoryRetriever = $categoryRetriever;
        $this->productRetriever  = $productRetriever;
        $this->customerHelper    = $customerHelper;
        $this->sgBase            = $base;
    }

    /**
     * @inheritdoc
     */
    public function getCategories($action, $shopNumber, $traceId, $limit = null, $offset = null, array $uids = [])
    {
        $categories = [];
        $rawCats    = $this->getCategoriesRaw($limit, $offset, $uids);
        foreach ($rawCats as $category) {
            $categories[] = $category->asArray();
        }

        return $categories;
    }

    /**
     * @inheritdoc
     */
    public function getCategoriesRaw($limit = null, $offset = null, array $uids = [])
    {
        return $this->categoryRetriever->getCategories($limit, $offset, $uids);
    }

    /**
     * @inheritdoc
     */
    public function getItems($action, $shopNumber, $traceId, $limit = null, $offset = null, array $uids = [])
    {
        $products = [];
        foreach ($this->getItemsRaw($limit, $offset, $uids) as $product) {
            $products[] = $product->asArray();
        }

        return $products;
    }

    /**
     * @inheritdoc
     */
    public function getItemsRaw($limit = null, $offset = null, array $uids = [])
    {
        return $this->productRetriever->getItems($limit, $offset, $uids);
    }

    /**
     * @inheritdoc
     */
    public function getCustomer($user, $pass)
    {
        $customer = $this->getCustomerRaw($user, $pass);

        return $customer->toArray();
    }

    /**
     * @inheritdoc
     */
    public function getCustomerRaw($user, $pass)
    {
        return $this->customerHelper->getCustomer($user, $pass);
    }

    /**
     * @inheritdoc
     */
    public function checkCart(array $cart)
    {
        $this->sgBase->loadArray($cart['cart']);
        $items = [];
        foreach ($this->checkCartRaw($this->sgBase) as $key => $value) {
            if (is_array($value)) {
                foreach ($value as $cartObject) {
                    if (method_exists($cartObject, 'toArray')) {
                        $items[$key][] = $cartObject->toArray();
                    } else {
                        $items[$key][] = $cartObject;
                    }
                }
            } elseif (method_exists($value, 'toArray')) {
                $items[$key] = $value->toArray();
            } else {
                $items[$key] = $value;
            }
        }

        return $items;
    }

    /**
     * Rewrite cart object, effectively injecting it into the system
     *
     * @inheritdoc
     */
    public function checkCartRaw($cart)
    {
        $this->sgBase->loadArray($cart->toArray());

        return $this->cartHelper->loadSupportedMethods();
    }
}
