<?php

/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

namespace Shopgate\Export\Test\Integration\Helper;

use Magento\Customer\Model\AccountManagement;
use Magento\Customer\Model\CustomerFactory;
use Magento\Framework\App\Config\ScopePool;
use Shopgate\Base\Test\Bootstrap;
use Shopgate\Base\Test\Integration\Db\ConfigManager;

/**
 * @coversDefaultClass \Shopgate\Export\Helper\Customer
 */
class CustomerTest extends \PHPUnit_Framework_TestCase
{
    /** @var CustomerFactory */
    protected $customerFactory;
    /** @var ConfigManager */
    protected $cfgManager;
    /** @var \Magento\Customer\Model\Customer[] */
    protected $customers;
    /** @var  ScopePool */
    protected $scopePool;
    /** @var \Shopgate\Export\Model\Service\Export */
    private $class;

    /**
     * Load object manager for initialization
     */
    public function setUp()
    {
        $objectManager         = Bootstrap::getObjectManager();
        $this->cfgManager      = new ConfigManager;
        $this->class           = $objectManager->create('Shopgate\Export\Model\Service\Export');
        $this->customerFactory = $objectManager->create('Magento\Customer\Model\CustomerFactory');
        $this->scopePool       = $objectManager->create('Magento\Framework\App\Config\ScopePool');
    }

    /**
     * Test that we can set a password on a user
     * and authenticate
     *
     * @covers ::getCustomer
     */
    public function testGetAuthenticatedCustomer()
    {
        $customer = $this->createCustomer();
        $customer->setData('confirmation', false);
        $customer->save();
        $sgCustomer = $this->class->getCustomerRaw($customer->getEmail(), $customer->getPassword());
        $result     = is_object($sgCustomer) && $sgCustomer->getMail() === $customer->getEmail();

        $this->assertTrue($result);
    }

    /**
     * Helps create a magento customer
     *
     * @param null|string $user
     * @param null|string $pass
     *
     * @return \Magento\Customer\Model\Customer
     * @throws \Exception
     */
    private function createCustomer($user = null, $pass = null)
    {
        if (!$pass) {
            $pass = 'test' . rand(0, 99999);
        }

        if (!$user) {
            $user = $pass . '@shopgate.com';
        }

        /** @var \Magento\Customer\Model\Customer $customer */
        $customer = $this->customerFactory->create();
        $customer->setData('email', $user);
        $customer->setData('firstname', 'Test');
        $customer->setData('lastname', 'Tester');
        $customer->setPassword($pass);
        $customer->save();

        return $this->customers[] = $customer;

    }

    /**
     * @covers ::getCustomer
     */
    public function testNonAuthenticatedCustomer()
    {
        $exceptionCode = \ShopgateLibraryException::PLUGIN_WRONG_USERNAME_OR_PASSWORD;
        $this->setExpectedException(
            'ShopgateLibraryException',
            \ShopgateLibraryException::getMessageFor($exceptionCode),
            $exceptionCode
        );
        $customer = $this->createCustomer();

        $this->class->getCustomerRaw($customer->getEmail(), 'wrong_password');
    }

    /**
     * Test if customer throws a non confirmed exception. This test may be
     * incomplete as we are having issues with proper codePool setup.
     *
     * @covers ::getCustomer
     */
    public function testConfirmRequiredCustomer()
    {
        $this->markTestSkipped('Skipping test as we are having issues with code pool');

        $exceptionCode = \ShopgateLibraryException::PLUGIN_CUSTOMER_ACCOUNT_NOT_CONFIRMED;
        $this->setExpectedException(
            'ShopgateLibraryException',
            \ShopgateLibraryException::getMessageFor($exceptionCode),
            $exceptionCode
        );
        /** @var \Magento\Customer\Model\Customer $customer */
        $customer = $this->createCustomer();
        $customer->setData('confirmation', '12345');
        $customer->save();
        $this->scopePool->getScope('websites')->setValue(AccountManagement::XML_PATH_IS_CONFIRM, 1);
        $this->class->getCustomerRaw($customer->getEmail(), $customer->getPassword());
    }

    /**
     * Reset the customer in the database
     */
    public function tearDown()
    {
        /**
         * @var \Magento\Framework\Registry $registry
         */
        $registry = Bootstrap::getObjectManager()->get('\Magento\Framework\Registry');
        $registry->register('isSecureArea', true, true);

        foreach ($this->customers as $customer) {
            $customer->delete();
        }
    }
}
