<?php
/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

namespace Shopgate\Export\Test\Integration\Helper\Product;

use Magento\Store\Model\StoreManagerInterface;
use Shopgate\Base\Test\Bootstrap;
use Shopgate\Export\Helper\Product\Retriever;

/**
 * @coversDefaultClass \Shopgate\Export\Helper\Product\Retriever
 */
class RetrieverTest extends \PHPUnit_Framework_TestCase
{
    /** @var StoreManagerInterface */
    protected $storeManager;
    /** @var Retriever */
    protected $class;

    /**
     * Init store manager + current class
     */
    public function setUp()
    {
        $objManager         = Bootstrap::getObjectManager();
        $this->storeManager = $objManager->create('Magento\Store\Model\StoreManagerInterface');
        $this->class        = $objManager->create('Shopgate\Export\Helper\Product\Retriever');
    }

    /**
     * Check if the created product can be pulled
     * individually by UID
     *
     * @param   int $uid
     *
     * @covers ::getItems
     * @dataProvider uidProvider
     */
    public function testGetProductsByUid($uid)
    {
        $sgProducts = $this->class->getItems(null, null, [$uid]);

        foreach ($sgProducts as $sgProduct) {
            /** @var \Shopgate\Export\Model\Export\Product $sgProduct */
            $this->assertEquals($uid, $sgProduct->getUid());
        }
    }

    /**
     * Provider for product uids
     *
     * @return array
     */
    public function uidProvider()
    {
        return [[21], [23]];
    }

    /**
     * @param $expected - expected number of items in collection
     * @param $limit
     * @param $offset
     *
     * @covers ::getItems
     * @dataProvider limitProvider
     */
    public function testGetItemsLimit($expected, $limit, $offset)
    {
        $sgProducts = $this->class->getItems($limit, $offset, []);
        $this->assertCount($expected, $sgProducts);
    }

    /**
     * Provider for limits and offsets
     *
     * @return array
     */
    public function limitProvider()
    {
        return [
            'first 5'             => [5, 5, 0],
            'next 3'              => [3, 3, 5],
            'non-existing offset' => [0, 5, 10000]
        ];
    }
}
