<?php
/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

namespace Shopgate\Export\Test\Integration\Model\Export;

use Shopgate\Base\Api\Config\SgCoreInterface;
use Shopgate\Base\Test\Bootstrap;
use Shopgate\Base\Test\Integration\Db\ConfigManager;

/**
 * @coversDefaultClass Shopgate\Export\Model\Export\Utility
 */
class UtilityTest extends \PHPUnit_Framework_TestCase
{
    /** @var  \Shopgate\Export\Model\Export\Utility */
    protected $class;
    /** @var  ConfigManager */
    protected $cfgManager;

    /**
     * Set up
     */
    public function setUp()
    {
        $this->cfgManager = new ConfigManager;
        $objManager       = Bootstrap::getObjectManager();
        $this->class      = $objManager->create('Shopgate\Export\Model\Export\Utility');
    }

    /**
     * Tests our url parser that inserts user:pass
     * into the given link
     *
     * @param string $expected
     * @param string $url
     * @param string $user
     * @param string $password
     *
     * @covers ::parseUrl
     * @dataProvider parseUrlProvider
     */
    public function testParseUrl($expected, $url, $user, $password)
    {
        $this->cfgManager->setConfigValue(SgCoreInterface::PATH_HTUSER, $user);
        $this->cfgManager->setConfigValue(SgCoreInterface::PATH_HTPASS, $password);

        $returned = $this->class->parseUrl($url);

        $this->assertEquals($expected, $returned);
    }

    /**
     * @return array
     */
    public function parseUrlProvider()
    {
        return [
            [
                'returned url' => 'http://user:pass@shopgate.com',
                'initial url'  => 'http://shopgate.com',
                'user'         => 'user',
                'password'     => 'pass',
            ],
            [
                'returned url' => 'http://shopgate.com',
                'initial url'  => 'http://shopgate.com',
                'user'         => '',
                'password'     => 'pass',
            ],
            [
                'returned url' => 'http://shopgate.com',
                'initial url'  => 'http://shopgate.com',
                'user'         => 'user',
                'password'     => '',
            ],
            [
                'returned url' => 'https://user:pass@shopgate.com',
                'initial url'  => 'https://shopgate.com',
                'user'         => 'user',
                'password'     => 'pass',
            ],
        ];
    }

    /**
     * Remove database entries
     */
    public function tearDown()
    {
        $this->cfgManager->removeConfigs();
    }
}