<?php
/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

namespace Shopgate\Export\Test\Unit\Helper\Export\Product;

use Magento\Catalog\Model\Product;
use Magento\Framework\TestFramework\Unit\Helper\ObjectManager;

/**
 * @coversDefaultClass \Shopgate\Export\Helper\Product\Utility
 */
class HelperTest extends \PHPUnit_Framework_TestCase
{

    /**
     * @var ObjectManager
     */
    private $objectManager;
    /**
     * @var \Shopgate\Export\Helper\Product\Utility
     */
    private $helper;

    /**
     * Load object manager for initialization
     */
    public function setUp()
    {
        $this->objectManager = new ObjectManager($this);
        $this->helper        = $this->objectManager->getObject('Shopgate\Export\Helper\Product\Utility');
    }

    /**
     * @param $expected  - expected mapped type from shopgate_library
     * @param $inputType - type to map
     *
     * @covers ::mapInputType
     *
     * @dataProvider mapInputTypeProvider
     */
    public function testMapInputType($expected, $inputType)
    {
        $mappedType = $this->helper->mapInputType($inputType);
        $this->assertEquals($expected, $mappedType);
    }

    /**
     * @return array
     */
    public function mapInputTypeProvider()
    {
        return [
            'mapping field to text'          => ['text', 'field'],
            'mapping area to area'           => ['area', 'area'],
            'mapping select to select'       => ['select', 'select'],
            'mapping drop_down to select'    => ['select', 'drop_down'],
            'mapping radio to select'        => ['select', 'radio'],
            'mapping checkbox to select'     => ['select', 'checkbox'],
            'mapping multiple to select'     => ['select', 'multiple'],
            'mapping multi to select'        => ['select', 'multi'],
            'mapping date to date'           => ['date', 'date'],
            'mapping date_time to datetime'  => ['datetime', 'date_time'],
            'mapping time to time'           => ['time', 'time'],
            'checking undefined type to map' => [false, 'unknown']
        ];
    }

    /**
     * @param float  $expected
     * @param string $priceType
     * @param float  $optionPrice
     * @param float  $productPrice
     *
     * @covers ::getOptionValuePrice
     * @covers ::getInputValuePrice
     * @covers       Shopgate\Export\Model\Export\Product::setInputs
     *
     * @dataProvider getOptionValuePriceProvider
     */
    public function testGetOptionValuePrice($expected, $priceType, $optionPrice, $productPrice)
    {
        $optionMock = $this->getMockBuilder(Product\Option::class)
                           ->disableOriginalConstructor()
                           ->setMethods(['getPriceType', 'getPrice'])
                           ->getMock();

        $optionMock->expects($this->once())
                   ->method('getPriceType')
                   ->will($this->returnValue($priceType));

        $optionMock->expects($this->once())
                   ->method('getPrice')
                   ->will($this->returnValue($optionPrice));

        $productMock = $this->getMockBuilder(Product::class)
                            ->disableOriginalConstructor()
                            ->setMethods(['getFinalPrice'])
                            ->getMock();

        $productMock->expects($this->any())
                    ->method('getFinalPrice')
                    ->will($this->returnValue($productPrice));

        $calculateOptionPrice = $this->helper->getOptionValuePrice($optionMock, $productMock);
        $this->assertEquals($expected, $calculateOptionPrice);
    }

    /**
     * @return array (expected value, price type, option price, product price)
     */
    public function getOptionValuePriceProvider()
    {
        return [
            'percentage based options' => [0.2, 'percent', 2.00, 10.00],
            'fixed price options'      => [2.00, 'fixed', 2.00, 10.00]
        ];
    }

    /**
     * @param array  $relatedIds
     * @param string $type
     *
     * @covers ::createRelationProducts
     * @covers       Shopgate\Export\Model\Export\Product::setRelations
     *
     * @dataProvider createRelationProductsProvider
     */
    public function testCreateRelationProducts($relatedIds, $type)
    {
        $relationModel = $this->helper->createRelationProducts($relatedIds, $type);
        $this->assertInstanceOf(\Shopgate_Model_Catalog_Relation::class, $relationModel);
        $this->assertEquals($type, $relationModel->getType());
        $this->assertEquals($relatedIds, $relationModel->getValues());
    }

    /**
     * @return array (related ids, relation type)
     */
    public function createRelationProductsProvider()
    {
        return [
            '2 upsell relations'     => [[1, 2], 'upsell'],
            'empty upsell relations' => [[], 'upsell']
        ];
    }
}
