<?php
/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain
 * unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

namespace Shopgate\Export\Test\Unit\Model\Export;

use Magento\ConfigurableProduct\Model\Product\Type\Configurable;
use Magento\Framework\TestFramework\Unit\Helper\ObjectManager;
use \Magento\Framework\App\Config;
use Magento\GroupedProduct\Model\Product\Type\Grouped;
use Shopgate\Base\Api\Config\CoreInterface;
use Magento\Catalog\Model\Product;
use Shopgate\Export\Model\Config\Source\Description;
use Magento\Catalog\Model\Product\Type\Simple;

/**
 * @coversDefaultClass \Shopgate\Export\Model\Export\Product
 */
class ProductTest extends \PHPUnit_Framework_TestCase
{

    /**
     * @var ObjectManager
     */
    private $objectManager;

    /**
     * @return \PHPUnit_Framework_MockObject_MockObject
     */
    private function getProductDouble()
    {
        $productDouble = $this
            ->getMockBuilder(Product::class)
            ->disableOriginalConstructor()
            ->getMock();

        return $productDouble;
    }

    /**
     * Load object manager for initialization
     */
    public function setUp()
    {
        $this->objectManager = new ObjectManager($this);
    }

    /**
     * @param $expectedDescription
     * @param $descConfig
     * @param $longDesc
     * @param $shortDesc
     *
     * @covers ::setDescription
     * @dataProvider descriptionProvider
     */
    public function testSetDescription($expectedDescription, $descConfig, $longDesc, $shortDesc)
    {
        $this->markTestIncomplete('Expected strings not yet returned by the stub');
        $configValueStub = $this->getMockBuilder(Config\Value::class)
                                ->disableOriginalConstructor()
                                ->getMock();

        $configValueStub->method('getValue')
                        ->will($this->returnValue($descConfig));

        $scopeConfigStub = $this->getMockBuilder(CoreInterface::class)
                                ->disableOriginalConstructor()
                                ->getMock();

        $scopeConfigStub->method('getConfigByPath')
                        ->will($this->returnValue($configValueStub));

        $productStub = $this->getProductDouble();

        $productStub->method('getDescription')
                    ->will($this->returnValue($longDesc));

        $productStub->method('getShortDescription')
                    ->will($this->returnValue($shortDesc));

        /** @var \Shopgate\Export\Model\Export\Product $exportModel */
        $exportModel = $this->objectManager->getObject(
            'Shopgate\Export\Model\Export\Product',
            [
                'scopeConfig' => $scopeConfigStub
            ]
        );

        $exportModel->setItem($productStub)->setDescription();

        $this->assertEquals($expectedDescription, $exportModel->getDescription());
    }

    /**
     * @param string $expectedDisplayType
     * @param string $productType
     *
     * @covers ::setDisplayType
     * @dataProvider displayTypeProvider
     */
    public function testdisplayType($expectedDisplayType, $productType)
    {
        $productStub = $this->getProductDouble();

        $productStub->method('getTypeId')
                    ->will($this->returnValue($productType));

        /** @var \Shopgate\Export\Model\Export\Product $exportModel */
        $exportModel = $this->objectManager->getObject(
            'Shopgate\Export\Model\Export\Product'
        );

        $exportModel->setItem($productStub)->setDisplayType();

        $this->assertEquals($expectedDisplayType, $exportModel->getDisplayType());
    }

    /**
     * @param $productType
     *
     * @covers ::setAttributeGroups
     * @dataProvider productTypeProvider
     */
    public function testEmptyAttributeGroups($productType)
    {
        $productStub = $this->getProductDouble();

        $productStub->method('getTypeId')
                    ->will($this->returnValue($productType));

        /** @var \Shopgate\Export\Model\Export\Product $exportModel */
        $exportModel = $this->objectManager->getObject(
            'Shopgate\Export\Model\Export\Product'
        );

        $exportModel->setItem($productStub)->setAttributeGroups();

        $this->assertEmpty($exportModel->getAttributeGroups());
    }

    /**
     * @return array
     */
    public function productTypeProvider()
    {
        return [
            [Grouped::TYPE_CODE],
            [Product\Type::TYPE_SIMPLE],
            [Product\Type::TYPE_BUNDLE],
            [Product\Type::TYPE_VIRTUAL]
        ];
    }

    /**
     * @return array
     */
    public function displayTypeProvider()
    {
        return [
            'grouped product'      => ['list', Grouped::TYPE_CODE],
            'configurable product' => ['select', Configurable::TYPE_CODE],
            'default type'         => ['simple', 'simple']
        ];
    }

    /**
     * @return array
     */
    public function descriptionProvider()
    {
        return [
            'default long description' => ['long', 'test', 'long', 'short'],
            'return short description' => ['short', Description::ID_SHORT_DESCRIPTION, 'long', 'short']
        ];
    }
}
